//=========================================================================
//
//	FILE:				CP_HelloWorld_View.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class to display the line "Hello World from CPLAT". 
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 8 $
//						Latest Checkout:  $Date: 2005-09-10 20:29:59 -0700 (Sat, 10 Sep 2005) $
//
//=========================================================================

	// HelloWorld
#include "CP_HelloWorld_View.h"

	// CPLAT_Framework
#include "CP_Enums.h"
#include "CP_Graphics.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_HelloWorld_View
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The subview handler that contains this pane.
*	@param		inUpperLeft			Upper left corner of pane in releation to its superview.
*	@param		inContentSize		Content size of the pane.
*	
*	@exception	none
*/
CP_HelloWorld_View::CP_HelloWorld_View( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize)
		: CP_View( inSuperview, inUpperLeft, inContentSize, true, GetFlags() | CPLAT::CP_E_ViewFlags_SupportsDragAndDrop )
{
}

//=========================================================================
//	FUNCTION:	~CP_HelloWorld_View
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CP_HelloWorld_View::~CP_HelloWorld_View() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark === Drawing ===
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the view needs to update the visual appearance of itself. In
*	this demo we use the CP_Graphics class to display the line "Hello World from CPLAT"
*	in the default font for a view.
*
*	@param		inCanvas			The CP_Canvas to draw into.
*	@param		inUpdateArea		The area that needs to be drawn.
*	
*	@exception	none
*
*	@return		void
*/
void CP_HelloWorld_View::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& inUpdateArea )
{
	CPLAT::CP_Rect			localBounds = GetLocalBounds();
	CPLAT::CP_String		text( CP_T("Hello World from CPLAT") );
	CPLAT::CP_Graphics		graphics( inCanvas );
	CPLAT::CP_FontSpec		fontSpec;

	graphics.DrawText( fontSpec, text, localBounds, CPLAT::CP_E_TextAlign_HorCenter, CPLAT::CP_E_TextAlign_VerMiddle );
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
