//=========================================================================
//
//	FILE:				CP_NibXML_Application.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Application class for the CP_CP_NibXML demo app.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 10 $
//						Latest Checkout:  $Date: 2005-08-30 20:32:29 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// CP_CP_NibXML
#include "CP_NibXML_App.h"

	// CPLAT_Framework
#include "CP_Alerts.h"
#include "CP_Application.tmpl.h"
#include "CP_BevelButton.h"
#include "CP_ChasingArrows.h"
#include "CP_CheckBox.h"
#include "CP_CheckGroupBox.h"
#include "CP_CPClassNames.h"
#include "CP_Clock.h"
#include "CP_Data_ResourceTypes.h"
#include "CP_Debugging.h"
#include "CP_DisclosureTriangle.h"
#include "CP_DocumentWindow.h"
#include "CP_EditField.h"
#include "CP_Environment.h"
#include "CP_Exceptions.h"
#include "CP_IconControl.h"
#include "CP_ImageWell.h"
#include "CP_LittleArrows.h"
#include "CP_New.h"
#include "CP_View.h"
#include "CP_PictureControl.h"
#include "CP_PersistManager.h"
#include "CP_Placard.h"
#include "CP_PopupArrow.h"
#include "CP_PopupGroupBox.h"
#include "CP_PopupMenu.h"
#include "CP_ProgressBar.h"
#include "CP_PushButton.h"
#include "CP_RadioButton.h"
#include "CP_RadioGroup.h"
#include "CP_RoundButton.h"
#include "CP_Relevance.h"
#include "CP_ScrollBar.h"
#include "CP_Separator.h"
#include "CP_Slider.h"
#include "CP_StaticText.h"
#include "CP_String.h"
#include "CP_TabView.h"
#include "CP_TextGroupBox.h"
#include "CP_TextView.h"
#include "CP_Types.h"
#include "CP_Window.h"
#include "CP_WindowHeader.h"
#include "CP_WindowManager.h"
#include "CP_XMLDocument.h"
#include "CP_Utility.h"
#include "CP_IBClassToCPXML.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_NibXML_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
CP_NibXML_Application::CP_NibXML_Application()
{
		// register the persist object creators
	RegisterPersist( CPLAT::CP_BevelButton,  		"CPLAT::CP_BevelButton" );
	RegisterPersist( CPLAT::CP_CheckBox,  			"CPLAT::CP_CheckBox" );
	RegisterPersist( CPLAT::CP_ChasingArrows,  		"CPLAT::CP_ChasingArrows" );
	RegisterPersist( CPLAT::CP_CheckGroupBox,  		"CPLAT::CP_CheckGroupBox" );
	RegisterPersist( CPLAT::CP_Clock,  				"CPLAT::CP_Clock" );
	RegisterPersist( CPLAT::CP_DisclosureTriangle,  "CPLAT::CP_DisclosureTriangle" );
	RegisterPersist( CPLAT::CP_DocumentWindow,  	"CPLAT::CP_DocumentWindow" );
	RegisterPersist( CPLAT::CP_EditField,  			"CPLAT::CP_EditField" );
	RegisterPersist( CPLAT::CP_IconControl,  		"CPLAT::CP_IconControl" );
	RegisterPersist( CPLAT::CP_ImageWell,  			"CPLAT::CP_ImageWell" );
	RegisterPersist( CPLAT::CP_LittleArrows,  		"CPLAT::CP_LittleArrows" );
	RegisterPersist( CPLAT::CP_PictureControl,  	"CPLAT::CP_PictureControl" );
	RegisterPersist( CPLAT::CP_Placard,  			"CPLAT::CP_Placard" );
	RegisterPersist( CPLAT::CP_PopupArrow,  		"CPLAT::CP_PopupArrow" );
	RegisterPersist( CPLAT::CP_PopupMenu,  			"CPLAT::CP_PopupMenu" );
	RegisterPersist( CPLAT::CP_PopupGroupBox,  		"CPLAT::CP_PopupGroupBox" );
	RegisterPersist( CPLAT::CP_ProgressBar,  		"CPLAT::CP_ProgressBar" );
	RegisterPersist( CPLAT::CP_PushButton,  		"CPLAT::CP_PushButton" );
	RegisterPersist( CPLAT::CP_RadioButton,  		"CPLAT::CP_RadioButton" );
	RegisterPersist( CPLAT::CP_RadioGroup,  		"CPLAT::CP_RadioGroup" );
	RegisterPersist( CPLAT::CP_Relevance,  			"CPLAT::CP_Relevance" );
	RegisterPersist( CPLAT::CP_RoundButton,  		"CPLAT::CP_RoundButton" );
	RegisterPersist( CPLAT::CP_ScrollBar,  			"CPLAT::CP_ScrollBar" );
	RegisterPersist( CPLAT::CP_Separator,  			"CPLAT::CP_Separator" );
	RegisterPersist( CPLAT::CP_Slider,  			"CPLAT::CP_Slider" );
	RegisterPersist( CPLAT::CP_StaticText,  		"CPLAT::CP_StaticText" );
	RegisterPersist( CPLAT::CP_TabView,  			"CPLAT::CP_TabView" );
	RegisterPersist( CPLAT::CP_TextView,  			"CPLAT::CP_TextView" );
	RegisterPersist( CPLAT::CP_TextGroupBox,  		"CPLAT::CP_TextGroupBox" );
	RegisterPersist( CPLAT::CP_View,  				"CPLAT::CP_View" );
	RegisterPersist( CPLAT::CP_Window,  			"CPLAT::CP_Window" );
	RegisterPersist( CPLAT::CP_WindowHeader,  		"CPLAT::CP_WindowHeader" );

	#if TARGET_OS_MAC
		CPLAT::RegisterKnownIBClasses();
	#endif
}

//=========================================================================
//	FUNCTION:	CP_NibXML_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CP_NibXML_Application::~CP_NibXML_Application() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === AboutBox ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetAboutBoxDescription
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Return the string that describes the purpose of the application in the
*	AboutBox.
*	
*	@exception	none
*
*	@return		CP_String const		The description, displayed in the about box.
*/
CPLAT::CP_String CP_NibXML_Application::GetAboutBoxDescription() const
{
	CPLAT::CP_String		returnDescription;
	
	returnDescription = "A simple CPLAT application to display a window created from a nib and converted to CPLAT XML.";
	
	return( returnDescription );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Commands ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleOpenApplication
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the application is starting up. We take this opportunity to
*	create our initial window and views. In this demo, we create the window
*	from an XML resource.
*	
*	@exception	none
*
*	@return		void
*/
void CP_NibXML_Application::HandleOpenApplication()
{
	try
	{
		CPLAT::CP_Window				*window = NULL;

		window = (CPLAT::CP_Window *) CPLAT::CP_PersistManager::Instance()->Create( 1000, CPLAT::kXMLDataResourceType );	
		CPLAT::CP_ThrowIfNULL(window);
		
		CPLAT::CP_MenuBarManager::Instance()->GetCurrentMenuBar()->SetMainWindow( window );

		CPLAT::CP_WindowManager::Instance()->Reposition( window, NULL, CPLAT::kCP_E_WindowPos_WindowCascadeOnMainScreen );
		
			// connect the popup mennu commands
		UInt16									itemCounter = 0;
		
		for( itemCounter = 0; itemCounter < 5; itemCounter++ )
			Connect( 2000 + itemCounter, &CP_NibXML_Application::HandlePopupCommand, CPLAT::CP_Command::CP_E_CH_AlwaysEnable );

		if( window != NULL )
			window->Show();
	}
	catch(...)
	{
		CP_ReportError( CPLAT::CP_String("An error has occurred."), CPLAT::CP_String("An error parsing our xml document has occcurrred") );
	}
}

//=========================================================================
//	FUNCTION:	HandleNew
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Handle the new menu command. This will call the HandleOpenApplication()
*	method to create the main window.
*
*	@param		inOutStatus				Returns CP_noErr.
*	@param		inCommandNo				Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_NibXML_Application::HandleNew( CPLAT::OSStatus& inOutStatus,
	CPLAT::CP_CommandNo& /*inCommandNo*/ )
{
	HandleOpenApplication();
	
	inOutStatus = CP_noErr;
}

//=========================================================================
//	FUNCTION:	HandlePopupCommand
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Call when the user selects an item from one of the popup menus. In this
*	demo we will show an alert with the message "Popup menu selected" and then
*	the command number.
*
*	@return		inOutStatus			Not used.
*	@return		inCommandNo			The command number associated with the selected item.
*	
*	@exception	none
*
*	@return		void
*/
void CP_NibXML_Application::HandlePopupCommand( CPLAT::OSStatus& /*inOutStatus*/,
	CPLAT::CP_CommandNo& inCommandNo )
{
	CPLAT::CP_String				alertMessage( "Popup menu selected: " );
	CPLAT::CP_String				alertDescription = CPLAT::CP_Utility::CP_SInt32ToString( inCommandNo );

	CPLAT::CP_Alerts::CP_OSAlert::ShowAlert( alertMessage, alertDescription );
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32
