//=========================================================================
//
//	FILE:				CP_SignalSlot_App.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Application class for the CP_SignalSlot demo.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 5 $
//						Latest Checkout:  $Date: 2005-08-30 20:33:24 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// Precompile header support
#include "CP_Precomp.h"

	// CP_SignalSlot;
#include "CP_SignalSlot_App.h"

	// CPLAT_Framework
#include "CP_Alerts.h"
#include "CP_Application.tmpl.h"
#include "CP_BevelButton.h"
#include "CP_EditField.h"
#include "CP_MenuBarManager.h"
#include "CP_ProgressBar.h"
#include "CP_ScrollBar.h"
#include "CP_Slider.h"
#include "CP_StaticText.h"
#include "CP_Window.h"
#include "CP_WindowManager.h"

	// ViewIDs
const UInt32 kCP_SliderViewID = CP_FOUR_CHAR_CODE('SLID');
const UInt32 kCP_ScrollBarViewID = CP_FOUR_CHAR_CODE('SCRL');
const UInt32 kCP_EditFieldViewID = CP_FOUR_CHAR_CODE('EDIT');

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_SignalSlot_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*	
*	@exception	none
*/
CP_SignalSlot_Application::CP_SignalSlot_Application()
{
}

//=========================================================================
//	FUNCTION:	~CP_SignalSlot_Application
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
CP_SignalSlot_Application::~CP_SignalSlot_Application() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === AboutBox ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	GetAboutBoxDescription
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Return the string that describes the purpose of the application in the
*	AboutBox.
*	
*	@exception	none
*
*	@return		CP_String const		The description, displayed in the about box.
*/
CPLAT::CP_String CP_SignalSlot_Application::GetAboutBoxDescription() const
{
	CPLAT::CP_String		returnDescription;
	
	returnDescription = "A simple CPLAT application to show signals and slots.";
	
	return( returnDescription );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Commands ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleNewCommand
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Override the New command handler to create new windows. Since this application
*	is using a NULL document policy, we don't normally have the New menu item
*	enabled.
*
*	@return		inOutStatus			Return the OSStatus of this operation here.
*	@return		inCommandNo			The command being executed, in this case
*									CP_CMD_New.
*	
*	@exception	none
*
*	@return		void
*/
void CP_SignalSlot_Application::HandleNewCommand( CPLAT::OSStatus& inOutStatus,
	CPLAT::CP_CommandNo& /*inCommandNo*/ )
{
	CPLAT::OSStatus			status = CP_noErr;
	
	HandleOpenApplication();

	inOutStatus = status;
}

//=========================================================================
//	FUNCTION:	HandleOpenApplication
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the application is starting up. We take this opportunity to
*	create our initial window and views. In this demo, we create several views
*	and show how to interconnect them using signals and slots.
*	
*	@exception	none
*
*	@return		void
*/
void CP_SignalSlot_Application::HandleOpenApplication()
{
	try
	{
		CPLAT::CP_Point					windowLocation( 50, 50 );
		CPLAT::CP_Size					windowSize( 300, 250 );
		CPLAT::CP_String				windowTitle( "Signal Slot Demo" );
		CPLAT::CP_DocumentWindowType	documentWindowType;
		
		fWindow = CP_new CPLAT::CP_Window();
		fWindow->Create( windowLocation, windowSize, windowTitle, documentWindowType );

		CPLAT::CP_MenuBarManager::Instance()->GetCurrentMenuBar()->SetMainWindow( fWindow );
	
		CPLAT::CP_WindowManager::Instance()->Reposition( fWindow, NULL, CPLAT::kCP_E_WindowPos_WindowCascadeOnMainScreen );

		CPLAT::CP_Point					viewLocation( 0, 0 );
		CPLAT::CP_Size					viewSize( 300, 150 );
		CPLAT::CP_ProgressBar			*progressBarOne = NULL;
		CPLAT::CP_Slider				*sliderOne = NULL;
		CPLAT::CP_ScrollBar				*scrollBarOne = NULL;
		CPLAT::CP_EditField				*editFieldOne = NULL;
		CPLAT::CP_String				editFieldText;
		CPLAT::CP_DataValueSignal		*valueChangedSignal = NULL;
		CPLAT::CP_StaticText			*staticTextDescription = NULL;
		CPLAT::CP_String				description;
		
		viewLocation.Set( 10, 10 );
		viewSize.Set( 250, 16 );
		description = "CP_View to CP_View connections";
		staticTextDescription = CP_new CPLAT::CP_StaticText( fWindow, viewLocation, viewSize, description );

			// slider
		viewLocation.Set( 10, 30 );
		viewSize.Set( 150, 24 );
		sliderOne = CP_new CPLAT::CP_Slider( fWindow, viewLocation, viewSize, 0, 0, 100 );
		sliderOne->SetID( kCP_SliderViewID );
		
			// scroll bar
		viewLocation.Set( 10, 60 );
		viewSize.Set( 150, 16 );
		scrollBarOne = CP_new CPLAT::CP_ScrollBar( fWindow, viewLocation, viewSize, 50, 0, 100 );
		scrollBarOne->SetID( kCP_ScrollBarViewID );

		viewLocation.Set( 220, 50 );
		viewSize.Set( 50, 22 );
		editFieldOne = CP_new CPLAT::CP_EditField( fWindow, viewLocation, viewSize, editFieldText, true );
		editFieldOne->AttachFilter( CP_new CPLAT::CP_KeyInputFilter_Numeric );
		editFieldOne->SetID( kCP_EditFieldViewID );

			// so we can tell if the slider or scroll bar changes values, connect up
			// to the value changed signal
		CPLAT::CP_DataValueSlot			editFieldValueSlot = editFieldOne->FindSlot( kCP_Slot_SetValue );
		valueChangedSignal = sliderOne->FindSignal( kCP_Signal_ValueChanged );
		valueChangedSignal->connect( editFieldValueSlot );

		valueChangedSignal = scrollBarOne->FindSignal( kCP_Signal_ValueChanged );
		valueChangedSignal->connect( editFieldValueSlot );

		CPLAT::CP_DataValueSlot			sliderSetValueSlot = sliderOne->FindSlot( kCP_Slot_SetValue );
		CPLAT::CP_DataValueSlot			scrollBarSetValueSlot = scrollBarOne->FindSlot( kCP_Slot_SetValue );

		valueChangedSignal = editFieldOne->FindSignal( kCP_Signal_ValueChanged );
		valueChangedSignal->connect( sliderSetValueSlot );
		valueChangedSignal->connect( scrollBarSetValueSlot );
		
			// connecct to the return signal to update the arrow
		editFieldOne->FindSignal( kCP_Signal_TextFieldExit )->connect( CPLAT::CP_SigSlot::slot(this, &CP_SignalSlot_Application::HandleReturn) );

			// set the value of the slider to 1/2 way, to update the scroll bar and edit field
		sliderOne->SetValue( 50 );

		CPLAT::CP_BevelButton			*bevelButtonOne = NULL;
		CPLAT::CP_S_ViewContentInfo		contentInfo = { CPLAT::kCP_E_ViewContent_TextOnly };

		viewLocation.Set( 10, 120 );
		viewSize.Set( 250, 16 );
		description = "CP_View to user defined";
		staticTextDescription = CP_new CPLAT::CP_StaticText( fWindow, viewLocation, viewSize, description );

		viewLocation.Set( 50, 150 );
		viewSize.Set( 150, 40 );
		description = "Click Me";
		contentInfo.fContentType = CPLAT::kCP_E_ViewContent_TextOnly;
		bevelButtonOne = CP_new CPLAT::CP_BevelButton( fWindow, viewLocation, viewSize, description, contentInfo, 1000 );
		bevelButtonOne->FindSignal( kCP_Signal_Clicked )->connect( CPLAT::CP_SigSlot::slot(this, &CP_SignalSlot_Application::ControlClicked) );

		fWindow->Show();
	}
	catch(...)
	{
	}
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Slots ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	ControlClicked
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot called when the user clicks on the click me button. This will post
*	an alert to acknowledge that the signal has been handled.
*
*	@return		inValue				Holds a CP_S_ControlClickedInfo structure that
*									describes information about the sending control
*									and value from the control that was clicked.
*	@return		inOutStatus			Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_SignalSlot_Application::ControlClicked( CPLAT::CP_DataValueHolder& /*inValue*/,
	CPLAT::OSStatus& /*inOutStatus*/ )
{
	CPLAT::CP_String					description( "Button clicked" );
	CPLAT::CP_String					message( "This is called from the Click Me button." );
			
	CPLAT::CP_Alerts::CP_OSAlert::ShowAlert( description, message );
}

//=========================================================================
//	FUNCTION:	HandleReturn
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Slot called when the user presses return.
*
*	@return		inValue				Not used
*	@return		inOutStatus			Set to CP_noErr.
*	
*	@exception	none
*
*	@return		void
*/
void CP_SignalSlot_Application::HandleReturn( CPLAT::CP_DataValueHolder& /*inDataValue*/,
	CPLAT::OSStatus& inOutOSStatus )
{
	SInt32						value = 0;
	CPLAT::CP_EditField			*editField = NULL;
	CPLAT::CP_ScrollBar			*scrollBar = NULL;
	CPLAT::CP_Slider			*slider = NULL;
	
	if( fWindow != NULL )
	{
		editField = dynamic_cast<CPLAT::CP_EditField *>(fWindow->FindSubView(kCP_EditFieldViewID, true));
		scrollBar = dynamic_cast<CPLAT::CP_ScrollBar *>(fWindow->FindSubView(kCP_ScrollBarViewID, true));
		slider = dynamic_cast<CPLAT::CP_Slider *>(fWindow->FindSubView(kCP_SliderViewID, true));
		
		if( editField != NULL )
			value = editField->GetValue();
		
		if( scrollBar != NULL )
			scrollBar->SetValue( value );
			
		if( slider != NULL )
			slider->SetValue( value );
	}
	
	inOutOSStatus = CP_noErr;
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
