//=========================================================================
//
//	FILE:				CP_Simple_TV_DB_Doc.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_Document class to show the CP_TV_DataBrower in a
//						simple single column tree view.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 6 $
//						Latest Checkout:  $Date: 2005-09-05 19:25:45 -0700 (Mon, 05 Sep 2005) $
//
//=========================================================================

	// Document
#include "CP_Simple_TV_DB_Doc.h"

	// CPLAT_Framework
#include "CP_Debugging.h"
#include "CP_Document.tmpl.h"
#include "CP_DocumentWindow.h"
#include "CP_New.h"
#include "CP_MenuBarManager.h"
#include "CP_OSStatus.h"
#include "CP_Scroller.h"
#include "CP_TV_DataBrowser.h"
#include "CP_TV_DataBrowserHandlers.h"
#include "CP_TV_DataBrowserRow.h"
#include "CP_TV_DataBrowser_Defines.h"
#include "CP_Types.h"
#include "CP_Utility.h"
#include "CP_Window.h"
#include "CP_WindowManager.h"

const UInt16 kNumberRows = 50;
const UInt16 kNumberChildRows = 5;
const UInt16 kNumberColumns = 1;

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_Simple_TV_DataBrowser_Doc
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inContainer			The scriptable object that serves as a container
*									for this document. In this case it will be the
*									application object.
*	@param		inDocType			The documents default file type.
*	
*	@exception	none
*/
CP_Simple_TV_DataBrowser_Doc::CP_Simple_TV_DataBrowser_Doc( CPLAT::CP_ScriptableObject *inContainer, CPLAT::FourCharCode inDocType )
	: inherited( inContainer, inDocType )
{
	CreateStringData();
}

//=========================================================================
//	FUNCTION:	~CP_Simple_TV_DataBrowser_Doc
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*
*	@exception	none
*/
CP_Simple_TV_DataBrowser_Doc::~CP_Simple_TV_DataBrowser_Doc() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Creation ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	DocumentCreator
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when some action has requested we create a new document object. This
*	is usually called from the CP_DocumentFactoryManager when it needs to create
*	a new document.
*
*	@param		inContainer			The scriptable object that serves as a container
*									for this document. In this case it will be the
*									application object.
*	@param		inDocumentClass		The documents default file type.
*	
*	@exception	none
*
*	@return		void
*/
CPLAT::CP_Document_Base *CP_Simple_TV_DataBrowser_Doc::DocumentCreator( CPLAT::CP_ScriptableObject *inContainer,
	CPLAT::FourCharCode inDocumentClass )
{
	return( CP_new CP_Simple_TV_DataBrowser_Doc( inContainer, inDocumentClass ) );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Data Source ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CreateStringData
//	
//	VISIBLITY:	Private
//=========================================================================
/*!
*	Called to create some strings used to popuplate the table with dummy data.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Simple_TV_DataBrowser_Doc::CreateStringData()
{
		// create the list of strings for the table
	SInt32				rowCounter = 1;
	SInt32				columnCounter = 1;
	
	for( rowCounter = 1; rowCounter <= ((kNumberRows + 1) * kNumberChildRows); rowCounter++ )
	{
		for( columnCounter = 1; columnCounter <= kNumberColumns; columnCounter++ )
		{
			CPLAT::CP_String		rowText("Row ");
			rowText += CPLAT::CP_Utility::CP_SInt32ToString( rowCounter );
			
			rowText += " / Col ";
			rowText += CPLAT::CP_Utility::CP_SInt32ToString( columnCounter );

			fTableStrings.push_back( rowText );
		}
	}
}

//=========================================================================
//	FUNCTION:	GetData
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	CP_DataSource method called to get data for the passed in identifier.
*
*	@return		inID					The identifier of the data to be retrieved.
*	
*	@exception	none
*
*
*	@return		CP_DataValueHolder		Contains the returned string data.
*/
CPLAT::CP_DataValueHolder CP_Simple_TV_DataBrowser_Doc::GetData( SInt32 inID )
{
	CPLAT::CP_String				rowData = fTableStrings.at( inID - 1 );
	CPLAT::CP_DataValueHolder		returnData = CPLAT::CP_DataValueHolder( rowData );
	
	return( returnData );
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Main Window ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	HandleCreateDocumentUI
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Called when the document wants to create the main user interface for the
*	document. In this demo we create a window, with a CP_TextView in it.
*
*	@param		inPostCreateFlags		Not used.
*	
*	@exception	none
*
*	@return		void
*/
void CP_Simple_TV_DataBrowser_Doc::HandleCreateDocumentUI( SInt32 inPostCreateFlags )
{
	try
	{
		CPLAT::CP_DocumentWindow		*window = NULL;
		CPLAT::CP_Point					windowLocation( 10, 50 );
		CPLAT::CP_Size					windowSize( 400, 200 );
		CPLAT::CP_String				windowTitle( "Simple CP_TV_DataBrowser" );
		CPLAT::CP_DocumentWindowType	documentWindowType;
		CPLAT::CP_Scroller				*tableScroller = NULL;
		
		window = CP_new CPLAT::CP_DocumentWindow();
		window->Create( windowLocation, windowSize, windowTitle, documentWindowType );

		CPLAT::CP_MenuBarManager::Instance()->GetCurrentMenuBar()->SetMainWindow( window );

		CPLAT::CP_Point					viewLocation( 0, 0 );
		CPLAT::CP_Size					viewSize( 400, 200 );
		SInt32							counter = 1;
		SInt32							childRowCounter = 1;
		SInt32							colCounter = 1;
		CPLAT::CP_Size					scrollUnit( 20, 20 );
		CPLAT::CP_S_TV_DB_Properties	simpleDataBrowserProperties;
		CPLAT::CP_TV_DataBrowser		*simpleDataBrowser = NULL;
		UInt32							nextID = 1;
		CPLAT::CP_TV_DataBrowserRow		*nextRow = NULL;
		CPLAT::CP_TV_DataBrowserRow		*childRow = NULL;
		UInt32							rowID = 1;
		
		tableScroller = CP_new CPLAT::CP_Scroller( window, viewLocation, viewSize );

		viewLocation.Set( 0, 0 );
		viewSize.Set( 400 - 16, 200 - 16 );
		simpleDataBrowserProperties.fNumberColumns = 1;
		simpleDataBrowser = CP_new CPLAT::CP_TV_DataBrowser( tableScroller, viewLocation, viewSize, simpleDataBrowserProperties );
		simpleDataBrowser->SetDataSource( this );
		simpleDataBrowser->SetID( 1234 );
		
		tableScroller->SetScrollingView( simpleDataBrowser );
		simpleDataBrowser->SetScrollUnit( scrollUnit );

			// set up the tables content handlers
		CPLAT::CP_S_TV_DB_ContentHandlerInfo		stringContentHandlerInfo( CPLAT::kCP_Content_String, CP_new CPLAT::CP_TV_DataBrowser_CH_String() );
		simpleDataBrowser->RegisterHandler( stringContentHandlerInfo );

		simpleDataBrowser->SetColumnContentType( 1, CPLAT::kCP_Content_String );

		simpleDataBrowser->SetColumnWidth( -1, 1, 250 );
		simpleDataBrowser->SetRowHeight( -1, 20 );
		
			// now add the data to the table view
		for( counter = 1; counter <= kNumberRows; counter++ )
		{
			nextRow = CP_new CPLAT::CP_TV_DataBrowserRow( simpleDataBrowser, rowID );
			simpleDataBrowser->AddRootRow( nextRow );
			rowID++;
			
				// set the cells data identifier for each cell in this row
			for( colCounter = 1; colCounter <= 1; colCounter++ )
			{
				CPLAT::CP_S_TV_DB_CellData			cellData = nextRow->GetCellData( colCounter );
				cellData.fCellID = nextID;
				nextRow->SetCellData( colCounter, cellData );
				
				nextID++;
			}

				// add the rows to the row created above as child rows
			for( childRowCounter = 1; childRowCounter <= kNumberChildRows; childRowCounter++ )
			{
				childRow = CP_new CPLAT::CP_TV_DataBrowserRow( simpleDataBrowser, rowID );
				nextRow->AddChild( childRow );
				rowID++;
				
					// set the cells data identifier for each cell in this row
				for( colCounter = 1; colCounter <= 1; colCounter++ )
				{
					CPLAT::CP_S_TV_DB_CellData			cellData = nextRow->GetCellData( colCounter );
					cellData.fCellID = nextID;
					childRow->SetCellData( colCounter, cellData );
					
					nextID++;
				}
			}
		}

			// associate our main window as the documents main window.
		SetMainWindow( window );
		
		CPLAT::CP_WindowManager::Instance()->Reposition( window, NULL, CPLAT::kCP_E_WindowPos_WindowCascadeOnMainScreen );

		window->Show();
	}
	catch(...)
	{
	}
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
