//=========================================================================
//
//	FILE:				CP_UIClasses_Tab_BackView.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class that acts as a placeholder for other
//						views in the tabbed view.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 7 $
//						Latest Checkout:  $Date: 2005-08-30 20:34:07 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// CP_UIClasses
#include "CP_UIClasses_Tab_BackView.h"

	// CPLAT_Framework
#include "CP_Color.h"
#include "CP_Graphics.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	CP_UIClasses_Tab_BackView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The subview handler that contains this pane.
*	@param		inUpperLeft			Upper left corner of pane in releation to its superview.
*	@param		inContentSize		Content size of the pane.
*	
*	@exception	none
*/
CP_UIClasses_Tab_BackView::CP_UIClasses_Tab_BackView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize)
		: CP_View( inSuperview, inUpperLeft, inContentSize, true )
{
}

//=========================================================================
//	FUNCTION:	~CP_UIClasses_Tab_BackView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*
*	@exception	none
*/
CP_UIClasses_Tab_BackView::~CP_UIClasses_Tab_BackView() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the view needs to update the visual appearance of itself. In
*	this demo we use the CP_Graphics class to display a border around the view
*	to visualize its location and size in the tab view.
*
*	@param		inCanvas			The CP_Canvas to draw into.
*	@param		inUpdateArea		The area that needs to be drawn.
*	
*	@exception	none
*
*	@return		void
*/
void CP_UIClasses_Tab_BackView::OnDraw( CPLAT::CP_Canvas& /*inCanvas*/,
	CPLAT::CP_Region& /*inUpdateArea*/ )
{
	// we don't draw anything, this is just a placeholder for other views
	// in the tab
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	SetForeBackColors
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	MacOS - Sets up the foreground and background colors of the view to match the containing
*	tab view. This is only needed and called on platforms that still need to setup
*	the background colors for AppearanceManager.
*
*	@exception	none
*
*	@return		void
*/
void CP_UIClasses_Tab_BackView::SetForeBackColors()
{
	GetSuperview()->SetForeBackColors();
}

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
