//=========================================================================
//
//	FILE:				Undo_TestView.cpp
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class to draw items to demonstrate undo/redo.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 5 $
//						Latest Checkout:  $Date: 2005-08-30 20:34:20 -0700 (Tue, 30 Aug 2005) $
//
//=========================================================================

	// UndoActions
#include "Undo_TestView.h"

	// CPLAT_Framework
#include "CP_Color.h"
#include "CP_Enums.h"
#include "CP_Graphics.h"

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Construction / Destruction ==
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	Undo_TestView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Constructor.
*
*	@param		inSuperview			The subview handler that contains this pane.
*	@param		inUpperLeft			Upper left corner of pane in releation to its superview.
*	@param		inContentSize		Content size of the pane.
*	
*	@exception	none
*/
Undo_TestView::Undo_TestView( CPLAT::CP_View *inSuperview,
	const CPLAT::CP_Point& inUpperLeft, const CPLAT::CP_Size& inContentSize)
		: CP_View( inSuperview, inUpperLeft, inContentSize, true )
{
}

//=========================================================================
//	FUNCTION:	~Undo_TestView
//	
//	VISIBLITY:	Public
//=========================================================================
/*!
*	Destructor.
*	
*	@exception	none
*/
Undo_TestView::~Undo_TestView() throw()
{
}

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Drawing ===
#pragma mark -
#endif

//=========================================================================
//	FUNCTION:	OnDraw
//	
//	VISIBLITY:	Protected
//=========================================================================
/*!
*	Called when the view needs to update the visual appearance of itself. In
*	this demo we use the CP_Graphics class to draw rectangles.
*
*	@param		inCanvas			The CP_Canvas to draw into.
*	@param		inUpdateArea		The area that needs to be drawn.
*	
*	@exception	none
*
*	@return		void
*/
void Undo_TestView::OnDraw( CPLAT::CP_Canvas& inCanvas, CPLAT::CP_Region& /*inUpdateArea*/ )
{
	CPLAT::CP_Rect			localBounds = GetLocalBounds();
	CPLAT::CP_Graphics		graphics( inCanvas );
	CPLAT::CP_Color			darkOrchidColor = CPLAT::CP_Color::GetColor( CPLAT::CP_Color::INDEX_darkorchid );
	CPLAT::CP_ARGBColor		graphicsDarkOrchidColor( darkOrchidColor.GetRGBColor() );

	graphics.SetStrokeColor( graphicsDarkOrchidColor );
	graphics.SetFillColor( graphicsDarkOrchidColor );

	graphics.FillRect( localBounds );
}

#if TARGET_OS_MAC

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Mac Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Win Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_WIN32

#if TARGET_OS_LINUX

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === Linux Specific ===
#pragma mark -
#endif

#endif	// TARGET_OS_LINUX
