//=========================================================================
//
//	FILE:				CP_Bitmap.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class for dealing with platform bitmaps.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_Bitmap
#define _H_CP_Bitmap
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_Palette.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Bitmap
*	@brief		Manages a platform native bitmap/pixmap format.
*/
class CP_EXPORT CP_Bitmap
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_Bitmap();
	//@}

	//! @name Getters
	//@{
				SInt16					GetColorDepth();
				CP_Palette				*GetPalette();

				SInt32					GetRowBytes();
				char 					*GetPixelsPtr();
				
				CP_RGBColor				GetPixelColor(
												SInt16 inCol,
												SInt16 inRow );
											
				SInt16					Width();
				SInt16					Height();

				SInt16					GetNumberBitPlanes();
				SInt16					GetPixelSize();
	//@}

	//! @name Lockers
	//@{
				void					Lock();
				void					Unlock();
	//@}

	//! @name Setters
	//@{
				void					SetPixel(
												SInt16 inCol,
												SInt16 inRow,
												SInt16 inPaletteIndex );
				void					SetPixel(
												SInt16 inCol,
												SInt16 inRow,
												SInt32 inColor );
				void					SetPixelColor(
												SInt16 inCol,
												SInt16 inRow,
												CP_RGBColor& inColor );
				void					SetPalette( CP_Palette *inPalette );
	//@}

private:
				
	// class data
protected:
	CP_Palette							*fBitmapPalette;			/*! Palette used to represent the colors of this bitmap. */
	bool								fOwnsPixmap;				/*! Flag to indicate if this object owns the underlying OS bitmap/pixmap. */
	
	#if TARGET_OS_MAC
		PixMapHandle					fPixMap;					/*! The MacOS pix map for this object. */
	#endif
	
	#if TARGET_OS_WIN32
		BITMAP							fBitMapInfo;				/*! Windows information about this bitmap. */
		HBITMAP							fBitMap;					/*! Windows HBITMAP for this object. */
		BITMAPINFOHEADER				fBitmapInfoHeaderStruct;	/*! Contains information about the Windows bitmap. */
		BITMAPINFOHEADER				*fBitmapInfoHeader;			/*! Contains information about the Windows bitmap. */
		HANDLE            				fDIB;

		SInt32							*fColorTable;				/*! Contains the color table for the Windows bitmap. */
		char							*fBitmapBits;				/*! Points to the bits of the Windows bitmap. */
	#endif

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Constructor / Destructor
	//@{
										CP_Bitmap(
												PixMapHandle inPixMap,
												bool inOwnsPixmap = false );
	//@}
					
	//! @name Getters
	//@{
										/*!
										*	Returns the MacOS pix map that is used to represent this object.
										*	
										*	@exception	none
										*	
										*	@return		PixMapHandle		MacOS PixMapHandle for this bitmap.
										*/
				PixMapHandle			GetPixMapHandle()									{ return( fPixMap ); }
	//@}
#endif

#if TARGET_OS_WIN32
public:
	//! @name Constructor / Destructor
	//@{
										CP_Bitmap( HBITMAP inBitMap );
										CP_Bitmap(
												SInt16 inWidth,
												SInt16 inHeight,
												SInt16 inColorDepth,
												bool inIsGrayscale = false,
												OSGraphicsPort inDestPort = NULL,
												bool inTopDown = false );
										CP_Bitmap(
												SInt16 inWidth,
												SInt16 inHeight,
												SInt16 inColorDepth,
												CP_Palette *inPalette = NULL,
												OSGraphicsPort inDestPort = NULL,
												bool inTopDown = false );
	//@}

	//! @name Palette
	//@{
				void					CreateBitmapPalette(
												SInt16 inColorDepth,
												bool inIsGrayscale = false );

				void					SetPaletteEntry(
												SInt16 inIndex,
												const CP_RGBColor& inColor );
	//@}

	//! @name Getters
	//@{
										/*!
										*	Returns the Windows HBITMAP that is used to represent this object.
										*	
										*	@exception	none
										*	
										*	@return		HBITMAP			The Window HBITMAP for this object.
										*/
				HBITMAP					GetBitmap()											{ return( fBitMap ); }
	//@}

private:
	//! @name Creator
	//@{
				void					CreateEmptyBitmap(
												SInt16 inWidth,
												SInt16 inHeight,
												SInt16 inColorDepth,
												bool inIsGrayscale = false,
												OSGraphicsPort inDestPort = NULL,
												bool inTopDown = false );

				void					CreateEmptyBitmap(
												SInt16 inWidth,
												SInt16 inHeight,
												SInt16 inColorDepth,
												CP_Palette *inPalette = NULL,
												OSGraphicsPort inDestPort = NULL,
												bool inTopDown = false );
	//@}

	//! @name Getters
	//@{
				SInt32					GetBitmapBytesNeeded(
												SInt16 inWidth,
												SInt16 inHeight,
												SInt16 inColorDepth );
				SInt32					GetBitsBytesNeeded(
												SInt16 inWidth,
												SInt16 inHeight,
												SInt16 inColorDepth );
	//@}
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Bitmap


