//=========================================================================
//
//	FILE:				CP_DataSource.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Mix-in class for objects that serve as a data source
//						for various CPLAT objects.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 243 $
//						Latest Checkout:  $Date: 2005-07-24 20:18:42 -0700 (Sun, 24 Jul 2005) $
//
//=========================================================================
#ifndef _H_CP_M_DataSource
#define _H_CP_M_DataSource
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_M_ReferenceCounted.h"
#include "CP_Object.h"
#include "CP_SignalSlot.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

struct CP_S_DataSource_ChangedData
{
	SInt32						fID;							// Data source id of the cell
	CP_DataValueHolder			fData;
	SInt32						fUserData;
	
									CP_S_DataSource_ChangedData();
									CP_S_DataSource_ChangedData( const CP_S_DataSource_ChangedData& inChangedData );

	CP_S_DataSource_ChangedData&	operator =( const CP_S_DataSource_ChangedData& inChangedData );
};

/*!
*	@class		CP_DataSource
*	@brief		Mix-in class for objects that serve as a data source for various CPLAT objects
*/
class CP_EXPORT CP_DataSource : public CP_M_RefCounted, public CP_Object, public CP_SigSlot::CP_SlotObject
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_DataSource();
	//! Destructor.
	virtual								~CP_DataSource() throw();
	//@}
			
	//! @name Data
	//@{
	virtual	CP_DataValueHolder			GetData( SInt32 inID );
	virtual	CP_DataValueHolder			GetData(
												SInt32 inID,
												UInt32 inOptionalProperty );
	virtual	void						HandleChangedData( CP_S_DataSource_ChangedData& inChangedData );
	//@}

	//! @name Notifications
	//@{
	virtual	void						DataChanged( SInt32 inID = -1 );
	//@}

	//! @name Sorting
	//@{
	virtual	bool						Compare(
												SInt32 inIDLeft,
												SInt32 inIDRight,
												bool inSortUp );
	//@}

protected:

private:
			void						DataChangedSlot(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );

	// class data
protected:
		// Signals
	CP_DataValueSignal					fValueChangedSignal;	/*! Signal sent when the value of any item changes. */
	
		// slots
	CP_DataValueSlot					fDataChangedSlot;

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_M_DataSource

