//=========================================================================
//
//	FILE:				CP_FontSpec.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A simple class used to specify the properties of a font.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 268 $
//						Latest Checkout:  $Date: 2005-08-31 22:38:24 -0700 (Wed, 31 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_FontSpec
#define _H_CP_FontSpec
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_RGBColor.h"
#include "CP_String.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

enum
{
	kCP_UseFontMask = 0x0001,
	kCP_UseFaceMask = 0x0002,
	kCP_UseSizeMask = 0x0004,
	kCP_UseForeColorMask = 0x0008,
	kCP_UseBackColorMask = 0x0010,
	kCP_UseModeMask = 0x0020,
	kCP_UseJustMask = 0x0040,
	kCP_UseAllMask = 0x00FF,
	kCP_UseAddFontSizeMask = 0x0100
};

/*!
*	@class		CP_FontSpec
*	@brief		A simple class used to specify the properties of a font.
*/
class CP_EXPORT CP_FontSpec
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FontSpec();
										CP_FontSpec( const CP_FontSpec& inFontSpec );
	//! Destructor.
	virtual								~CP_FontSpec() throw();
	//@}

	//! @name Getters
	//@{
			SInt16						GetFlags()										{ return( fFlags ); }
			CP_String					GetName()										{ return( fName ); }
			SInt16						GetSize()										{ return( fSize ); }
			SInt16						GetStyle()										{ return( fStyle ); }
			SInt16						GetJustification()								{ return( fJustification ); }
			CP_RGBColor					GetForeColor()									{ return( fForeColor ); }
			CP_RGBColor					GetBackColor()									{ return( fBackColor ); }
	
			SInt16						GetOSFontNumber()								{ return( fOSFontNumber ); }
	//@}

	//! @name Operators
	//@{
			CP_FontSpec&				operator = ( const CP_FontSpec& inFontSpec );
	//@}

	//! @name Resources
	//@{
			void						Load( SInt16 inResourceID );
	//@}

	//! @name Setters
	//@{
			void						SetFlags( SInt16 inFlags )						{ fFlags = inFlags; }
			void						SetName( const CP_String& inName );
			void						SetSize( SInt16 inSize );
			void						SetStyle( SInt16 inStyle );
			void						SetMode( SInt16 inMode );
			void						SetJustification( SInt16 inJustification );
			void						SetForeColor( const CP_RGBColor& inForeColor );
			void						SetBackColor( const CP_RGBColor& inBackColor );

			void						SetFromSystemName( const CP_String& inFontName );

			void						SetOSFontNumber( SInt16 inOSFontNumber );
	//@}

protected:

	// class data
protected:
	SInt16								fFlags;				/*! Flags that indicate which field is valid. */
	CP_String							fName;				/*! Name of the font. */
	SInt16								fSize;				/*! Size for the font. */
	SInt16								fStyle;				/*! Style used on the font, bold, italic, etc. */
	SInt16								fMode;				/*! Transfer mode used with font. */
	SInt16								fJustification;		/*! Justification used. */
	CP_RGBColor							fForeColor;			/*! Foreground color. */
	CP_RGBColor							fBackColor;			/*! Background color. */

	SInt16								fOSFontNumber;
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	//! @name Misc
	//@{
			void						FillInControlFontStyleRec( ControlFontStyleRec& inFontStyleRec );

			void						SetMetaFontNumber(
												SInt16 inMetaFontNumber,
												SInt16 inFlags = 0 );
	//@}
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FontSpec

