/*
=========================================================================

	FILE:		CP_Gradient.h
	
				Copyright  2001-2003. Ken Stahlman. All Rights Reserved.
	
	PURPOSE:	Base class to handle construction and drawing of gradients
				to a CP_Canvas object.
	
	HISTORY:	1st Version - September 2001.
	
	NOTES:		- none -

	VERSION:	Current Revision: $Revision: 2 $
				Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $

=========================================================================
*/
#ifndef _H_CP_Gradient
#define _H_CP_Gradient
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Fixed.h"
#include "CP_NoCopy.h"
#include "CP_OffscreenCanvas.h"
#include "CP_Point.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

enum CP_E_GradientType
{
	CP_E_GradientType_TopToBottom,
	CP_E_GradientType_BottomToTop,
	CP_E_GradientType_LeftToRight,
	CP_E_GradientType_RightToLeft,
	CP_E_GradientType_Radial,
	CP_E_GradientType_Sunburst,
	CP_E_GradientType_InsetSunburst,
	CP_E_GradientType_Shapeburst,
	CP_E_GradientType_Angled
};

typedef struct
{
	SInt32			fRedDiff;
	SInt32			fGreenDiff;
	SInt32			fBlueDiff;
	SInt16			fRedIncrement;
	SInt16			fGreenIncrement;
	SInt16			fBlueIncrement;
}
CP_E_GradientColorInfo;

const SInt16 kCP_MaxGradientColors									= 5;

class CP_EXPORT CP_Gradient : public CP_NoCopy
{
public:
									// Contstruction / Destruction
								CP_Gradient();
	virtual						~CP_Gradient();

									// Getters
			CP_E_GradientType	GetGradiientType()										{ return( fGradientType ); }
			SInt16				GetGradientAngle()										{ return( fGradientAngle ); }
			SInt16				GetNumberGradients()										{ return( fGradientCount ); }

									// Drawing
			void				Draw(
										CP_Canvas& inDestCanvas,
										CP_Rect& inDestBounds,
										CP_Region& inDestRegion );

	virtual	void				BuildGradient(
										CP_Canvas& inDestCanvas,
										CP_Rect& inDestBounds,
										CP_Point& inCenterPoint );

									// Setters
			void				SetGradiientType( CP_E_GradientType inGradientType )	{ fGradientType = inGradientType; fNeedsBuilding = true; }
			void				SetGradientAngle( SInt16 inAngle )						{ fGradientAngle = inAngle; }
			void				SetColor(
										const SInt16 inIndex,
										const CP_RGBColor& inColour );

			void				SetGradientCount( SInt16 inGradientCount )				{ fGradientCount = inGradientCount; }

protected:

private:
			void				Init();

			SInt32				CalcGradientRange(
										CP_Rect& inBounds,
										CP_Point& inCenterPoint );
			void				CalcGradientColors(
										SInt32 inRange,
										CP_RGBColor& inOutColorOne,
										CP_RGBColor& inOutColorTwo,
										CP_E_GradientColorInfo *inOutGradientInfo );	
			void				AdjustColor(
										CP_RGBColor& inOutColor,
										CP_E_GradientColorInfo *inOutGradientInfo );
			void				CalcEnclosingBounds(
										CP_Rect& inOutBounds,
										CP_Point& inCenterPoint );	

			void				BuildRect(
										CP_Rect& inDestBounds,
										CP_Point& inCenterPoint,
										CP_E_GradientType inGradientType );
			void				BuildRadial(
										CP_Rect& inBounds,
										CP_Point& inCenterPoint );
			void				BuildSunburst(
										CP_Rect& inBounds,
										CP_Point& inCenterPoint,
										CP_E_GradientType inGradientType );
			void				BuildShapeburst(
										CP_Rect& inBounds,
										CP_Point& inCenterPoint );
			void				BuildAngled( CP_Rect& inBounds );

			UInt16				CP_FixedToColor( CP_Fixed inValue );

	// class data
protected:
	CP_E_GradientType			fGradientType;
	SInt16						fGradientCount;
	CP_RGBColor					fColors[ kCP_MaxGradientColors ];
	SInt16						fOffsets[ kCP_MaxGradientColors ];
	SInt16						fGradientAngle;
	CP_Region					fDestRegion;

    CP_OffscreenCanvas			*fGradientCanvas;
	
	bool						fNeedsBuilding;
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Gradient
