//=========================================================================
//
//	FILE:				CP_Malloc.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to wrap malloc / calloc / free functions.
//	
//	HISTORY:			1st Version - July 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev:$
//						Latest Checkout:  $Date:$
//
//=========================================================================
#ifndef _H_CP_Malloc
#define _H_CP_Malloc
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_Malloc
*	@brief		A class to wrap malloc / calloc / free functions.
*	
*				This class allocates memory blocks using malloc, and provides
*				for allocation in the constructorr, and deallocation in the destructor.
*/
class CP_EXPORT CP_Malloc
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Malloc();
										explicit CP_Malloc( UInt32 inSize );
										explicit CP_Malloc( CP_Malloc& inPtrBlock );
	//! Destructor.
										~CP_Malloc();
	//@}

	//! @name Copying
	//@{
				void					CopyTo(
												void *ioBuffer,
												UInt32 inNumBytes ) const;
				void					CopyFrom(
												const void *inBuffer,
												UInt32 inNumBytes );
	//@}

	//! @name Getters
	//@{
				char					*GetPtr() const;
	//@}

	//! @name Length
	//@{
				UInt32					Length() const;
				void					SetLength( UInt32 inLength );
	//@}

	//! @name Operators
	//@{
			CP_Malloc&					operator = (const CP_Malloc& inPtrBlock );
	//@}
				
	//! @name Testers
	//@{
										/*!
										*	Returns true if the memory block is not NULL.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if the memory block is not NULL.
										*/
				bool					IsValid() const								{ return( fBlock != NULL ); }

										/*!
										*	Returns true if this object owns the block of memory, and is therefore
										*	responsible for its deletion. Otherwise the memory block held by this object
										*	will not be deleted by this class.
										*	
										*	@exception	none
										*	
										*	@return		bool			Returns true if this object owns the memory handle.
										*/
				bool					OwnsPtr() const								{ return( fOwnsPtr ); }
	//@}

	//! @name Wrapping
	//@{
				void					Wrap(
											char *inExistingPtr,
											bool inOwnsPtr = true );

										/*!
										*	Notifies this class that it no longer owns the memory, and will not
										*	delete it. You are now responsible for deleting the memory to avoid a
										*	memory leak.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
				void					Detach()									{ fOwnsPtr = false; }
	//@}

private:
	//! @name Allocation
	//@{
				void					Allocate( UInt32 inSize );
				void					Deallocate();
	//@}

	// class data
protected:
	char								*fBlock;				/*! Points to the allocated memory. */
	bool								fOwnsPtr;				/*! Flag to indicate who is responsible for deleting the platforms memory. */

	UInt32								fActualBlockLength;		/*! Length of allocated memory. */
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Malloc



