//=========================================================================
//
//	FILE:				CP_PropertyList_DateTime.h
//	
//						Created by Jens Miltner on 01.12.04.
//	
//	PURPOSE:			CP_PropertyList date time.
//	
//	HISTORY:			1st Version - January 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 115 $
//						Latest Checkout:  $Date: 2005-03-24 16:55:37 -0800 (Thu, 24 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_PropertyList_DateTime
#define _H_CP_PropertyList_DateTime

#include "CP_PropertyList_Node.h"
#include "CP_Date.h"
#include "CP_Time.h"
#include "CP_DateTime.h"
#if CP_PropertyList_Uses_CoreFoundation
#include "CP_CFObject.h"
#endif

CPLAT_Begin_Namespace_CPLAT

namespace CP_PropertyList {

class CP_EXPORT DateTime : public Node, public CP_DateTime
#if CP_PropertyList_Uses_CoreFoundation
	,private CP_CFObject<CFDateRef>
#endif
{
	public:
		DateTime();
		explicit DateTime(const CP_GregorianDate& inGregorianDate, const CP_TimeZone* inTimeZone=NULL);
		explicit DateTime(const CP_DateTime& inDateTime);
		explicit DateTime(const CP_Date& inDate, const CP_Time& inTime);
		explicit DateTime(const CP_Date& inDate);
		explicit DateTime(const CP_Time& inTime);
		DateTime(const DateTime& inSrc);
	#if CP_PropertyList_Uses_CoreFoundation
		explicit DateTime(CFDateRef inDateRef, bool inRetain=true);
	#endif
		virtual ~DateTime() throw();
		
		virtual NodeType_t NodeType() const throw();

		static DateTime* FromXMLData(CP_InStream& inXMLDataStream);

		virtual void PrintToLog(CP_LogFile& inLogFile, SInt32 inLevel=0, SInt32 inIndentLevel=0) const;
	#if CP_PropertyList_Uses_CoreFoundation
		virtual CFTypeRef/*CFDateRef*/ GetNodeCFRef() const;
	#endif
	#if !CP_PropertyList_Uses_CoreFoundation
	protected:
		//! write XML data for this node only (i.e. without xml header, plist root, etc.)
		virtual void WriteNodeXMLData(CP_OutStream& inXMLDataStream, int inLeadingSpaces) const;
	#endif
		// class data:
	protected:
	};

} // end of namespace CP_PropertyList

CPLAT_End_Namespace_CPLAT

#endif // _H_CP_PropertyList_DateTime
