//=========================================================================
//
//	FILE:				CP_StColorStateSaver.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A stack based class to save and restore the colors
//						of a platforms OS graphics port.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_StColorStateSaver
#define _H_CP_StColorStateSaver
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_NoCopy.h"
#include "CP_Region.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

//=========================================================================
//	CP_StColorStateSaver
//		Use to save and restore the color state of a current QD port or Win
//		DC.
//=========================================================================
//!  CP_TextState
/*!
  class to save and restore the color state of the port in a CP_Canvas object.
  \sa CP_Canvas.
*/
class CP_EXPORT CP_StColorStateSaver : public CP_NoCopy
{
public:
										CP_StColorStateSaver();
										CP_StColorStateSaver( CP_Canvas& inCanvas );
										~CP_StColorStateSaver();
				
			void						Save();
			void						Restore();
	
			void						Default();

private:
								
	CP_Canvas							fCanvas;			/*! Canvas object we are saving information in. */
	
	#if TARGET_OS_MAC
		RGBColor						fSavedForeColor;	/*! Saved MacOS foreground color. */
		RGBColor						fSavedBackColor;	/*! Saved MacOS background color. */
	#endif

	#if TARGET_OS_WIN32
		COLORREF						fSavedBackColor;	/*! Saved Windows background color. */
	#endif

	#if TARGET_OS_LINUX
		GdkGCValues						fSavedGCValues;
	#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_StColorStateSaver

