//=========================================================================
//
//	FILE:				CP_StTextStateSaver.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A stack based class to save and restore the text state
//						of a platforms OS graphics port.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_StTextStateSaver
#define _H_CP_StTextStateSaver
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Canvas.h"
#include "CP_NoCopy.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

//========================================================================
//	CP_StTextStateSaver
//		Save and restore the text state for a CP_Canvas object.
//========================================================================
//!  CP_StTextStateSaver
/*!
  class to save and restore the text state of a CP_Canvas object.
  \sa CP_Canvas.
*/
class CP_EXPORT CP_StTextStateSaver : public CP_NoCopy
{
public:
											// Construction / destruction
										CP_StTextStateSaver( CP_Canvas& inCanvas );
										~CP_StTextStateSaver();

			void						Save();
			void						Restore();
										
			void						Default();
			
protected:
										
	CP_Canvas							fCanvas;		/*! Canvas object we are saving information about. */
	SInt16								fFont;			/*! Saved font number. */
	SInt16								fFontSize;		/*! Saved font size. */
	SInt16								fFontStyle;		/*! Saved font style. */
	SInt16								fTransferMode;	/*! Saved font transfer mode. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_StTextStateSaver


