//=========================================================================
//
//	FILE:				CP_SystemMemory.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Object to maintain information about the available memory
//						available from the platforms memory pool.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_SystemMemory
#define _H_CP_SystemMemory
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_NoCopy.h"
#include "CP_Types.h"

CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_SystemMemory : public CP_NoCopy
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_SystemMemory();
	//! Destructor.
	virtual								~CP_SystemMemory() throw();
	//@}
									
	//! @name Getters
	//@{
			SInt32						FreeMemory();
	//@}
			
	//! @name Testers
	//@{
			bool						IsMemoryAvailable( SInt32 inDesiredAmount );
			bool						IsMemoryLow();
	//@}
									
	// class data
protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

//========================================================================
// CP_GrowZone - A MacOS only class to handle management of a grow zone for
//		low memory situations. This class is only needed for MacOS classic.
//========================================================================
#if TARGET_OS_MAC
	#ifndef __CF_USE_FRAMEWORK_INCLUDES__
			// amount of extra memory to allocate for extra margin of available memory - 2k
		const SInt16 kCP_MemoryReserve_BufferExtra						= (1024 * 2);
		
		const UInt32 kCP_MemoryReserve_UserAlertResType					= CP_FOUR_CHAR_CODE('ALRT');
		
		const UInt32 kCP_MemoryReserve_UserAlertID						= 500;

			// Message sent to listeners to ask them to free memory
		//const CP_MESSAGE CP_MSG__GROW_ZONE_FREE_MEMORY						= 500;

		class CP_EXPORT CP_GrowZone /*: public CP_M_Idler, CP_M_Broadcaster*/
		{
		public:
			//! @name Constructor / Destructor
			//@{
			//! Constructor.
										CP_GrowZone( SInt32 inMemoryReserve );
			//! Destructor.
			virtual						~CP_GrowZone();
			//@}
											
			//! @name Instance
			//@{
			static CP_GrowZone			*Instance();
			//@}

			//! @name Setters
			//@{
										/*!
										*	Sets a flag to alert the user when memory is getting low.
										*	
										*	@exception	none
										*	
										*	@return		void
										*/
					void				AlertUser()								{ fAlertUser = true; }
			//@}
											
			//! @name Testers
			//@{
					bool				HasMemoryReserve();
			//@}
					
		protected:
											// Events
		//	virtual	void				OnIdleSelf( CP_Event& inEvent );

			//! @name Memory management
			//@{
					SInt32				HandleGrowZoneRecoverMemory( SInt32 inBytesNeeded );
			virtual SInt32				RecoverMemmory( SInt32 inBytesNeeded );

					SInt32				UseMemoryReserve();

			static pascal SInt32		CP_GrowZoneCallBack( SInt32 inBytesNeeded );
			//@}

			// class data
		protected:
			static CP_GrowZone			*fsInstance;			/*! Single instance of this class. */
			static GrowZoneUPP			gGrowZoneUPP;			/*! UPP used with the grow zone to manage memory. */

			Handle						fReservedMemory;		/*! Desired amount of memory to hold in reserve. */
			Size						fReservedMemorySize;	/*! Actual amount of memory being held in reserve. */
			bool						fAlertUser;				/*! Flag to indicate if we should alert the user when memory is low. */
		};
	#endif	// __CF_USE_FRAMEWORK_INCLUDES__
#endif	// __BUILD_MAC__

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_SystemMemory



