//=========================================================================
//
//	FILE:				CP_Types.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Definitions for the various types used in CPLAT.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 269 $
//						Latest Checkout:  $Date: 2005-09-05 19:20:19 -0700 (Mon, 05 Sep 2005) $
//
//=========================================================================
#ifndef _H_CP_Types
#define _H_CP_Types
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FrameworkFlags.h"

#if TARGET_OS_MAC
		// Apple includes
	#if TARGET_API_MAC_CARBON
		#ifdef __CF_USE_FRAMEWORK_INCLUDES__
			#include <Carbon/Carbon.h>
		#else
			#include <Carbon.h>
		#endif
	#else
		#include <MacTypes.h>
	#endif
#endif

#if TARGET_OS_WIN32
		// Win SDK
	#ifndef _WINDOWS_
	#include <windows.h>
	#endif
	#include <commctrl.h>
	#include <wctype.h>

		// Standard Library
	#include <memory>
	#include <string>
#endif

#if TARGET_OS_LINUX	
	#include <gtk/gtk.h>

		// Standard Library
	#include <memory>
	#include <string>
#endif

	// declare these out of the CPLAT namespace so they appear in the
	// global namespace for compability with Apple defines.
	// These define some Apple types we use under Windows and Linux if not
	// using the QuickTime headers.
#if TARGET_OS_WIN32 || TARGET_OS_LINUX
		// define the standard Apple defines for Windows and Linux
	typedef unsigned char							UInt8;
	typedef signed char								SInt8;
	typedef unsigned short							UInt16;
	typedef signed short							SInt16;
	typedef unsigned long							UInt32;
	typedef signed long								SInt32;

	#if TARGET_CC_MSVC && ! TARGET_CC_METROWERKS
		typedef   signed __int64					SInt64;
		typedef unsigned __int64					UInt64;
	#else
		typedef signed long long					SInt64;
		typedef unsigned long long					UInt64;
	#endif

	typedef SInt16                         		 	OSErr;

	#if TARGET_OS_WIN32
		typedef wchar_t                       		CPUniChar;
	#else
		typedef UInt16                        		CPUniChar;
	#endif
	
	typedef UInt32                          		CPUniCharCount;
#else
	typedef UInt16                        			CPUniChar;
	typedef UInt32                          		CPUniCharCount;
#endif	// TARGET_OS_WIN32 || TARGET_OS_LINUX

CPLAT_Begin_Namespace_CPLAT

//=========================================================================
// CPLAT framework version
//=========================================================================
#define __CPLAT_II__								0x00020000		// Version 2.0

//=========================================================================
// CPLAT Types
//=========================================================================
const char				MAX_CHAR_8 = 127;
const char				MIN_CHAR_8 = -128;

const unsigned char		MAX_UCHAR_8 = 255;

const SInt16			MAX_SINT_16	= 32767;
const SInt16			MIN_SINT_16	= -32768;

const SInt32			MAX_SINT_32 = 0x7FFFFFFF;	//  2,147,483,647
const SInt32			MIN_SINT_32 = 0x80000000;	// -2,147,483,648

const UInt32			MAX_UINT_32	 = 0xFFFFFFFF;

typedef SInt32			CP_CommandNo;

	// so we also have defined for the Win version
#if TARGET_OS_MAC
	#define CP_FOUR_CHAR_CODE(x)					(x)
#endif
#if TARGET_OS_WIN32
     #define CP_FOUR_CHAR_CODE(x)      	 			(((unsigned long) ((x) & 0x000000FF)) << 24) | (((unsigned long) ((x) & 0x0000FF00)) << 8) | (((unsigned long) ((x) & 0x00FF0000)) >> 8) | (((unsigned long) ((x) & 0xFF000000)) >> 24)
#endif
#if TARGET_OS_LINUX
     #define CP_FOUR_CHAR_CODE(x)      	 			(((unsigned long) ((x) & 0x000000FF)) << 24) | (((unsigned long) ((x) & 0x0000FF00)) << 8) | (((unsigned long) ((x) & 0x00FF0000)) >> 8) | (((unsigned long) ((x) & 0xFF000000)) >> 24)
#endif

#define CP_MakeSInt32(x,y)							((SInt32)(((SInt32)(x)) | ((SInt32)((SInt32)(y))) << 16))
#define CP_LoWord(x)								((SInt32)(x))
#define CP_HiWord(x)								((SInt32)(((SInt32)(x) >> 16) & 0xFFFF))

	// common clipboard data type
#if TARGET_OS_MAC
	#define kCP_Clipboard_TypeText					CP_FOUR_CHAR_CODE('TEXT')
	#define kCP_Clipboard_TypeUnicodeText			CP_FOUR_CHAR_CODE('utxt')
	#define kCP_Clipboard_TypePicture				CP_FOUR_CHAR_CODE('PICT')
#endif

#if TARGET_OS_WIN32
	#define kCP_Clipboard_TypeText					CF_TEXT
	#define kCP_Clipboard_TypeUnicodeText			CP_FOUR_CHAR_CODE('utxt')
	#define kCP_Clipboard_TypePicture				CF_BMP
#endif

#if TARGET_OS_LINUX
	#define kCP_Clipboard_TypeText					0x54455854 //CP_FOUR_CHAR_CODE('TEXT')
	#define kCP_Clipboard_TypeUnicodeText			0x75747874 //CP_FOUR_CHAR_CODE('utxt')
	#define kCP_Clipboard_TypePicture				0x50494354 //CP_FOUR_CHAR_CODE('PICT')
#endif

//=========================================================================
// Debugging support
//=========================================================================
#if TARGET_CC_MSVC
	#ifndef __FUNCTION__
		#define __FUNCTION__		""
	#endif
#endif

//=========================================================================
// File system
//=========================================================================
	// Note: These types correspond to the OSFileRef typdefed below and used
	// in the CP_File classes.
#if TARGET_OS_MAC
	const SInt16 kCP_BadFileRefNum					= -1;
#endif	// TARGET_OS_MAC

#if TARGET_OS_WIN32
	const HANDLE kCP_BadFileRefNum					= INVALID_HANDLE_VALUE;
#endif	// TARGET_OS_MAC

#if TARGET_OS_LINUX
	const SInt16 kCP_BadFileRefNum					= -1;
#endif	// TARGET_OS_MAC

#if TARGET_OS_LINUX
//=========================================================================
// Linux specific
//=========================================================================
const SInt32 kCP_GTK_EVENT							= CP_FOUR_CHAR_CODE('GTKE');

#endif	// TARGET_OS_WIN32

//=========================================================================
// Misc defines
//=========================================================================
	// definne NULL
#if __GNUC__ || TARGET_OS_LINUX
	#undef NULL
	#define NULL 0
#else
	#ifndef NULL
		#if !defined(__cplusplus) && (defined(__SC__) || defined(THINK_C))
			#define NULL ((void *) 0)
		#else
			#define NULL 0
		#endif
	#endif
	#ifndef nil
		#define nil NULL
	#endif
#endif

	// define our own min / max to avoid STL/MSVC defined mixups
#define CP_MIN(a,b)									(((a) < (b)) ? (a) : (b))
#define CP_MAX(a,b)									(((a) > (b)) ? (a) : (b))

#if TARGET_CC_MSVC
	#ifndef min
		#define min(a,b)							(((a) < (b)) ? (a) : (b))
		#define max(a,b)							(((a) > (b)) ? (a) : (b))
	#endif
#endif

#if TARGET_OS_MAC
	#ifndef topLeft
		#define topLeft(inRect)						(((Point *) &(inRect))[0])
	#endif

	#ifndef botRight
		#define botRight(inRect)					(((Point *) &(inRect))[1])
	#endif
#endif

#define CP_PI					3.141592653589793116
static inline double DegreesToRadians( double inDegrees )							{ return( ( inDegrees * CP_PI ) / 180.0 ); }
static inline double RadiansToDegrees( double inRadians )							{ return( inRadians * ( CP_PI / 180 ) ); }

const SInt16 kCP_PStyleString_MaxLength				= 255;
const SInt32 kCP_InvalidIndex						= -1;		// used to indicate that index are not valid
const SInt16 kCP_NoImageIndex						= -1;
		
#define	CP_CR										'\r'
#define	CP_LF										'\n'
#define	CP_CRLF										"\r\n"
#define	CP_TAB										'\t'

//=========================================================================
// Namespace support
//=========================================================================
	// Defines in the code for std:: due to different compilers use of the std namespace
	// If compiling for the following:
	//		CodeWarrior Mac/Windows			define
	//		CodeWarrior Linux				undefine
#if ENABLE_STD_NAMESPACE
	#define CP_USING_STD_NAMESPACE					using namespace std;
	#define CP_STD									std
#else
	#define CP_USING_STD_NAMESPACE
	#define CP_STD									
#endif
		
//=========================================================================
// Platform to CPLAT types
//=========================================================================
#if TARGET_OS_MAC
	#if TARGET_API_MAC_CARBON
		typedef Handle			OSMenuBar;				// Handle to Mac Menu Bar
		typedef MenuRef			OSMenu;					// Opaque reference to Mac Menu
		typedef WindowRef		OSWindow;				// Opaque reference to Mac Window
		typedef GrafPtr			OSGraphicsPort;			// Mac GrafPtr
		typedef Handle			OSHandle;				// Mac Handle
		typedef RgnHandle		OSRegion;				// Mac Region
		typedef ControlRef		OSControl;				// Opaque reference to Mac Control
		typedef Ptr				OSPtr;					// Mac memory Ptr
		typedef SInt16			OSFileRef;				// Mac file reference number
		typedef GWorldPtr		OSImageList;			// Mac GWorld containing images
		typedef CIconHandle		OSColorIcon;			// Mac CIcon
		typedef PicHandle		OSPicture;				// Mac Picture Handle
		typedef IconRef			OSIconRef;				// Mac IconRef
		typedef Handle			OSIconSuite;			// Mac Icon Suite Handle
		typedef FSVolumeRefNum 	OSVolume;				// Mac volume ref num
		typedef WindowClass 	OSWindowClass;			// Mac WindowClass
	#endif	// TARGET_API_MAC_CARBON
#endif

#if TARGET_OS_WIN32
	typedef HMENU			OSMenuBar;					// Windows HMENU handle
	typedef HMENU			OSMenu;						// Handle to HMENU Menu
	typedef HWND			OSWindow;					// Windows HWND handle
	typedef HDC				OSGraphicsPort;				// Windows HDC device context
	typedef HGLOBAL			OSHandle;					// Windows Handle
	typedef HRGN			OSRegion;					// Windows Region
	typedef HPEN			OSPen;						// Windows Pen
	typedef HBRUSH			OSBrush;					// Windows Brush
	typedef HWND			OSControl;					// Windows Control (a HWND child window)
	typedef HGLOBAL			OSPtr;						// Windows memory reference
	typedef HANDLE			OSFileRef;					// Win file reference handle
	typedef HICON			OSColorIcon;				// Win icon
	typedef HBITMAP			OSPicture;					// Windows HBitmap
	typedef HICON			OSIconRef;					// Win icon
	typedef HICON			OSIconSuite;				// Win icon
	#if UNICODE
		typedef std::wstring	OSVolume;				// Windows drive
	#else
		typedef std::string		OSVolume;				// Windows drive
	#endif
	typedef HIMAGELIST		OSImageList;				// Windows image list
	typedef UInt32 			OSWindowClass;			// Mac WindowClass
#endif

#if TARGET_OS_LINUX
	typedef GtkWidget		*OSMenuBar;					// Windows HMENU handle
	typedef GtkWidget		*OSMenu;					// Handle to Gtk menu widget
	typedef GtkWidget		*OSWindow;					// Gtk window widget
	typedef GdkGC			*OSGraphicsPort;			// Gdk graphic context
	typedef char **			OSHandle;					// Linux char **
	typedef GdkRegion		*OSRegion;					// Linux GdkRegion *
	typedef GtkWidget		*OSControl;					// Gtk control widget
	typedef short			OSFileRef;					// Linux file reference
	typedef char **			OSColorIcon;				// Linux xpm image data
	typedef char **			OSPicture;					// Linux xpm image data
	typedef char **			OSIconRef;					// Linux xpm image data
	typedef char **			OSIconSuite;				// Linux xpm image data
	typedef char *			OSPtr;						// char *
	typedef char *			OSVolume;
	typedef char **			OSImageList;				// Image list data
	typedef UInt32 			OSWindowClass;			// Mac WindowClass
#endif

//=====================================================================
// Unicode
//=========================================================================
#if ! defined(CP_T)
	#if UNICODE
		#define CP_T(x)                   L##x
	#else
		#define CP_T(x)                   x
	#endif
#endif

#if TARGET_OS_WIN32
	//=====================================================================
	// Windows specific
	//=========================================================================
	const SInt32 kCP_WINDOW_EVENT					= CP_FOUR_CHAR_CODE('WINE');

	#if TARGET_CC_MSVC
		#pragma warning(disable : 4068)		// unknown pragma
		#pragma warning(disable : 4138)		// /* outside comment
		#pragma warning(disable : 4200)		// empty array
		#pragma warning(disable : 4244)		// NULL to long type conversion
		#pragma warning(disable : 4291)		// operator new
		#pragma warning(disable : 4786)		// STL warnings
		#pragma warning(disable : 4800)		// BOOL / bool value change
	#endif

#endif	// TARGET_OS_WIN32

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_Types

