//=========================================================================
//
//	FILE:				CP_GetDirectoryDialog.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Presents the platforms GetDirectoryDialog.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 40 $
//						Latest Checkout:  $Date: 2005-01-01 21:50:04 -0800 (Sat, 01 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_GetDirectoryDialog
#define _H_CP_GetDirectoryDialog
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_FileSystemSpec.h"
#include "CP_FileSystemDialogsReply.h"

	// Standard Library
#include <vector>

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_GetDirectoryDialog
*	@brief		Presents the platforms GetDirectoryDialog.
*/
class CP_EXPORT CP_GetDirectoryDialog : public CP_FileSystemDialogs_Reply
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_GetDirectoryDialog();
	//! Destructor.
	virtual								~CP_GetDirectoryDialog() throw();
	//@}
			
	//! @name Getters
	//@{
										/*!
										*	Returns the list of selected directories.
										*	
										*	@exception	none
										*	
										*	@return		CP_FileSystemSpec_Array			Selected directories.
										*/
			CP_FileSystemSpec_Array		GetSelectedDirectoriesList()								{ return( fSelectedDirectories ); }
	//@}

protected:

	// class data
protected:
	CP_FileSystemSpec_Array				fSelectedDirectories;	/*! Selected directories by user */
			
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	//! @name Dialog handlers
	//@{
	virtual	NavDialogRef				CreateDialog(
												NavDialogCreationOptions& inOptions, 
												NavEventUPP inEventUPP,
												void *inUserData );

	virtual	void						ProcessReply();

	virtual	void						HandleUserAction(
												CP_FileSystemDialogs_Base *inSendingDialog,
												CP_E_NavDialogResult inNavDialogResult );
	//@}
#endif

#if TARGET_OS_WIN32
protected:
	//! @name Dialog handlers
	//@{
	virtual	void						RunPlatformDialog();

	static INT CALLBACK					CP_SelectDirectoryHook(
												HWND hWnd,
												UINT uMsg,
												LPARAM lParam,
												LPARAM lpData );
	//@}

private:
			void						RunPlatformDialog_Unicode();
			void						RunPlatformDialog_NonUnicode();

#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_GetDirectoryDialog

