//=========================================================================
//
//	FILE:				CP_ServerSocket.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Class to allow incoming socket connections.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 106 $
//						Latest Checkout:  $Date: 2005-03-21 15:03:06 -0800 (Mon, 21 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_ServerSocket
#define _H_CP_ServerSocket
#ifndef __BUILD_LINUX__
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Socket.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_ServerSocket
*	@brief		Class to allow incoming socket connections.
*/
class CP_EXPORT CP_ServerSocket : public CP_Socket
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ServerSocket();
										CP_ServerSocket(UInt32 inAddress, UInt16 inPort);
										CP_ServerSocket(const CP_NetworkAddress& inAddress, UInt16 inPort);
	//! Destructor.
	virtual								~CP_ServerSocket() throw();
	//@}

	//! @name Listing
	//@{
			void 						BeginListening( SInt16 queSize = SOMAXCONN );
			void 						BeginListening(
												UInt32 inAddress,
												UInt16 inPort,
												SInt16 queSize = SOMAXCONN );
			void 						BeginListening(
												const CP_NetworkAddress&  inAddress,
												UInt16 inPort,
												SInt16 queSize = SOMAXCONN );
	
			CP_Socket 					*Accept();
	//@}

private:
	//! @name Binding
	//@{
			void						BindTo(
												UInt32 inAddress,
												UInt16 inPort );
			void						BindTo(
												const CP_NetworkAddress&  inAddress,
												UInt16 inPort );
	//@}
	
											// Make private, because Connect should not be called on a server socket.
										CP_Socket::Connect;

	// class data
private:
	bool								fListening;		/*! Is the socket currently accepting connections. */

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	//! @name Create
	//@{
	virtual	CP_Socket 					*createSocket( int s );
	//@}
#endif

#if TARGET_OS_WIN32
protected:
	//! @name Create
	//@{
	virtual CP_Socket 					*createSocket( SOCKET s );
	//@}
#endif

#if TARGET_OS_LINUX
protected:
	//! @name Create
	//@{
	virtual CP_Socket 					*createSocket( int s );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_ServerSocket
