//=========================================================================
//
//	FILE:				CP_XMLHelpers.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A set of routines / class members to write data in
//						XML format.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 247 $
//						Latest Checkout:  $Date: 2005-07-30 19:06:38 -0700 (Sat, 30 Jul 2005) $
//
//=========================================================================

#ifndef _H_CP_XMLHelpers
#define _H_CP_XMLHelpers
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Application.h"
#include "CP_DocumentPolicy_None.h"
#include "CP_OSStatus.h"
#include "CP_Types.h"
#include "CP_UndoPolicy_None.h"
#include "CP_XMLDocument.h"
#include "CP_XMLWriter.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_XMLHelpers
*	@brief		A set of routines / class members to write data in XML format.
*
*	@ingroup	XML
*/
class CP_XMLHelpers
{
public:
#if TARGET_OS_MAC
	//! @name Getters
	//@{
	static	CPLAT::CP_String			GetControlTitle(
												OSControl inControl,
												ControlKind& inControlKind );
	//@}
#endif
	
	//! @name Reading
	//@{
	static 	CPLAT::CP_DataValueList		CreateDataValueList( CPLAT::CP_XMLDocNode *inContainingViewDocNode );

	static	bool						ReadRect(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												CPLAT::CP_Rect &outValue );
	static	bool						ReadHIRect(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												CPLAT::CP_Rect &outValue );
	static	bool						ReadString(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												CPLAT::CP_String &outValue );
	static	bool						ReadSInt16(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												SInt16& outValue );
	static	bool						ReadUInt16(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												UInt16& outValue );
	static	bool						ReadSInt32(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												SInt32& outValue );
	static	bool						ReadUInt32(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												UInt32& outValue );
	static	bool						ReadCharData(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												CPLAT::CP_String &outValue );
	static	bool						ReadBool(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												bool &outValue );
	static	bool						ReadBoolean(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												bool &outValue );
	static	bool						ReadOSType(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												OSType& outValue );
	static	bool						ReadRGBColor(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag,
												CP_RGBColor& outValue );
	//@}
	
//#if TARGET_OS_MAC
	//! @name Subviews
	//@{
	static	void						HandleSubviews(
												CPLAT::CP_XMLDocNode *inWindowNode,
												CPLAT::CP_XMLWriter_Element *inRootElement,
												CPLAT::CP_XMLWriter_Element *inParentElement );
	static	void						WriteSubviews(
												CPLAT::CP_XMLDocNode *inParentNode,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement );
	static	void						WriteViewsSubviews(
												CPLAT::CP_XMLDocNode *inParentNode,
												const CPLAT::CP_String& inIBClassName,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												CPLAT::CP_Rect& inOutBounds,
												bool& inOutInPortBounds );
	//@}

	//! @name Writing
	//@{
	static	void						ConvertWindowToXML( OSWindow inWindow );
	static	void						WriteWindow(
												OSWindow inWindow,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement );
	static	void						WriteControls(
												OSControl inParentControl,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_Point& inSuperLocation );
	static	void						WriteControl(
												OSControl inControl,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_Point& inSuperLocation );
	//@}

#if TARGET_OS_MAC
	//! @name Controls
	//@{
	static	void						WriteDefaultControlKind(
												OSControl inControl,
												CPLAT::CP_String& inClassName,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_Point& inSuperLocation,
												ControlKind& inControlKind );
	//@}
#endif

	//! @name Misc
	//@{
	static	CPLAT::CP_XMLWriter_Element	*WriteWidget(
												const CPLAT::CP_String& inWidgetName,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement );
	static CPLAT::CP_XMLWriter_Element *WriteSubviewsCount(
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												UInt32 inSubviewsCount );

	static	void						WriteRect(
												const CPLAT::CP_Rect& inRect,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WritePoint(
												const CPLAT::CP_Point& inSize,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteSize(
												const CPLAT::CP_Size& inSize,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteString(
												const CPLAT::CP_String& inString,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteSInt16(
												SInt16 inSInt16,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteUInt16(
												UInt16 inUInt16,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteSInt32(
												SInt32 inSInt32,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteUInt32(
												UInt32 inUInt32,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteBool(
												bool inBool,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteOSType(
												OSType inOSType,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );
	static	void						WriteRGBColor(
												const CP_RGBColor& inColor,
												CPLAT::CP_XMLWriter *inXMLWriter,
												CPLAT::CP_XMLWriter_Element *inParentElement,
												const CPLAT::CP_String& inClassName );

	static 	void						WriteXMLToFile(
												CPLAT::CP_XMLWriter_Element *inRootElement,
												const CPLAT::CP_FileSystemSpec &inFileSpec );
	//@}

	//! @name XMLResource
	//@{
	static	void						WriteData_MacXMLResource(
												const CPLAT::CP_HandleBlock& inData,
												CPLAT::CP_File& inFile,
												const CPLAT::CP_String& inResourceNaame,
												SInt16 inResourceID );
	static	void						WriteData_LinuxXMLResource(
												const CPLAT::CP_HandleBlock& inData,
												CPLAT::CP_File& inFile,
												const CPLAT::CP_String& inResourceNaame );
	//@}

	//! @name ChildNodes
	//@{
	static	CPLAT::CP_XMLDocNode		*FindChildNodeByFirstAttribute(
												CPLAT::CP_XMLDocNode *inNode,
												const CPLAT::CP_String& inDataTag );
	//@}

	// class data
protected:
		
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_XMLHelpers

