//=========================================================================
//
//	FILE:				CP_ATSUI_TextBlock.h
//	
//						Copyright  2005. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle display of text using ATSUI.
//	
//	HISTORY:			1st Version - February 2005.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 94 $
//						Latest Checkout:  $Date: 2005-02-20 20:03:45 -0800 (Sun, 20 Feb 2005) $
//
//=========================================================================
#ifndef _H_CP_ATSUI_TextBlock
#define _H_CP_ATSUI_TextBlock
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_Rect.h"
#include "CP_String.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_ATSUI_TextBlock
*	@brief		A class to draw a text block using MacOS ATSUI functions.
*/
class CP_EXPORT CP_ATSUI_TextBlock
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_ATSUI_TextBlock();
										CP_ATSUI_TextBlock(
												const CP_String& inText,
												const CP_Rect& inBounds,
												const CP_FontSpec& inFontSpec );
	//! Destructor.
	virtual								~CP_ATSUI_TextBlock() throw();
	//@}

	//! @name Drawing
	//@{
			void						DrawString(
												CPLAT::CP_Canvas& inDestCanvas,
												const CP_String& inText,
												const CP_Rect& inBounds,
												const CP_FontSpec& inFontSpec );

			#if TARGET_RT_MAC_MACHO
				void					DrawString(
												CPLAT::CP_Canvas& inDestCanvas,
												const CP_String& inText,
												const CP_Rect& inBounds,
												const CP_FontSpec& inFontSpec,
												const CP_Rect& inContextRefBonds );
			#endif
			
			void						DrawText();
			void						DrawText(
												const CP_String& inText,
												const CP_Rect& inBounds,
												const CP_FontSpec& inFontSpec );
			#if TARGET_RT_MAC_MACHO
			void						DrawText(
												CPLAT::CP_Canvas& inDestCanvas,
												const CP_String& inText,
												const CP_Rect& inBounds,
												const CP_FontSpec& inFontSpec,
												const CP_Rect& inContextRefBonds );
			#endif
	//@}

protected:
	//! @name Getters
	//@{
			OSStatus					ATSUI_GetOneAttribute(
												ATSUStyle iStyle,
												ATSUAttributeTag iTag,
												ByteCount iExpectedValueSize,
												ATSUAttributeValuePtr oValue );
			OSStatus					ATSUI_GetOneLayoutControl(
												ATSUTextLayout iLayout,
												ATSUAttributeTag iTag,
												ByteCount iExpectedValueSize,
												ATSUAttributeValuePtr oValue );

			Fract						ATSUI_FlushFactorFromTEAlignment( SInt16 alignment );

			ATSUFontID 					GetFontIDFromMacFontName( Str255 fontName );
	//@}

	//! @name Setters
	//@{
			OSStatus					ATSUI_SetBooleanAttribute(
												ATSUStyle iStyle,
												Boolean iSetting,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetFixedAttribute(
												ATSUStyle iStyle,
												Fixed iValue,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetFractAttribute(
												ATSUStyle iStyle,
												Fract iValue,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetIntAttribute(
												ATSUStyle iStyle,
												SInt32 iValue,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetShortAttribute(
												ATSUStyle iStyle,
												SInt16 iValue,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetPtrAttribute(
												ATSUStyle iStyle,
												void* iValue,
												ATSUAttributeTag iTag );

			OSStatus					ATSUI_SetBooleanLayoutControl(
												ATSUTextLayout iLayout,
												Boolean iValue,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetFixedLayoutControl(
												ATSUTextLayout iLayout,
												Fixed iValue,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetFractLayoutControl(
												ATSUTextLayout iLayout,
												Fract iValue,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetShortLayoutControl(
												ATSUTextLayout iLayout,
												SInt16 iValue,
												ATSUAttributeTag iTag );
			OSStatus					ATSUI_SetPtrLayoutControl(
												ATSUTextLayout iLayout,
												void *iValue,
												ATSUAttributeTag iTag );
	//@}


private:
			void						Init();

			void						InitATSUIStyle();

			void						SetFont(
												ATSUStyle theStyle,
												Str255 fontName );

			void						BreakTextInLines();
			
	// class data
private:
	CP_String							fText;
	CP_Rect								fBounds;	
	CP_FontSpec							fFontSpec;
	
	UniCharArrayPtr						fUnicodeText;				// the Text in Unicode
	UniCharCount						fTextLength;				// its length
	ATSUTextMeasurement					fHorLocation;				// where it starts drawing at x
	ATSUTextMeasurement					fVerLocation;				//                        and y
	ATSUTextMeasurement					fMaxAscent;					// maximum ascent for the whole text
	ATSUTextMeasurement					fMaxDescent;				// maximum descent for the whole text
	ATSUTextMeasurement					fLineHeight;				// its line height (maxAscent + maxDescent)
	ItemCount							fNumberOfLines;				// number of lines to draw
	UniCharArrayOffset					*fEndOfLines;				// positions of the end of line breaks
	ItemCount							fNumberOfRuns;				// number of style runs
	UniCharCount						*fRunLengths;				// array of the lengths of each style run
	ATSUStyle							*fStyles;					// array of styles
	ATSUTextLayout						fTextLayout;				// the resulting text layout
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_ATSUI_TextBlock

