//=========================================================================
//
//	FILE:				CP_CFBundle.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A class to handle a MacOS CFBundleRef.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 271 $
//						Latest Checkout:  $Date: 2005-09-06 19:24:58 -0700 (Tue, 06 Sep 2005) $
//
//=========================================================================
#ifndef _H_CP_CFBundle
#define _H_CP_CFBundle
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_CFURL.h"
#include "CP_NoCopy.h"
#include "CP_ResourcePtr.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_CFBundle
*	@brief		A class to handle a MacOS CFBundle.
*/
class CP_EXPORT CP_CFBundle
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_CFBundle();
	explicit							CP_CFBundle( CP_ResourcePtr<CFBundleRef> inBundleRef );
	explicit							CP_CFBundle( CFBundleRef inBundleRef );
										CP_CFBundle( const CP_CFBundle& inBundle );
	//! Destructor.
	virtual								~CP_CFBundle() throw();
	//@}
			
	//! @name Getters
	//@{
			CP_CFURL					GetURL() const;
			CP_CFURL					GetResourceURL() const ;
			CP_CFURL 					GetResourceURL(
												const CP_String& inFileName,
												const CP_String& inExtension ) const;

										/*!
										*	Return the MacOS CFBundleRef for this bundle.
										*	
										*	@exception	none
										*	
										*	@return		CFBundleRef		MacOS CFBundleRef for this bundle.
										*/
			CFBundleRef					GetOSBundle() const								{ return( fBundleRef ); }

										/*!
										*	Return the MacOS package creator code for the bundle.
										*	
										*	@exception	none
										*	
										*	@return		OSType			The MacOS page creator code for the bundle.
										*/
			OSType						GetPackageCreator()	const						{ return( fCreator ); }

										/*!
										*	Return the MacOS package type for the bundle.
										*	
										*	@exception	none
										*	
										*	@return		OSType			The MacOS package type for the bundle.
										*/
			OSType						GetPackageType() const							{ return( fType ); }
			CP_String					GetShortVersionString() const;
			CP_String					GetCopyrightString() const;
			CFTypeRef					GetValueForDictionaryKey( const CP_String& inKey ) const;
	//@}

	//! @name operators
	//@{
			CP_CFBundle&				operator =( const CP_CFBundle& inBundle );
	//@}

	//! @name Tester
	//@{
			bool						IsValid();
	//@}

protected:

	// class data
protected:
	CFBundleRef							fBundleRef;		/*! The bundle we manage. */
	OSType								fCreator;		/*! Creator code of the bundle. */
	OSType								fType;			/*! Type code of the bundle. */
};

CPLAT_End_Namespace_CPLAT

#endif  //TARGET_OS_MAC

#endif	// _H_CP_CFBundle

