//=========================================================================
//
//	FILE:				CP_CFType.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			A set of classes and routines for dealing with MacOS
//						CFTypeRef data.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 131 $
//						Latest Checkout:  $Date: 2005-03-28 19:43:25 -0800 (Mon, 28 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_CFType
#define _H_CP_CFType
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_ResourcePtr.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@function	CP_CFType_GetTypeID
*	@brief		Returns the CFTypeID.
*/
template <typename T> CFTypeID	CP_CFType_GetTypeID();

/*!
*	@function	CP_CFType_Get
*	@brief		Returns in inOutValue a CP_ResourcePtr for the value with the passed in key, and stored
*				in the passed in dictionary.
*/
template <typename T> bool CP_CFType_Get( CFDictionaryRef inDictionary, CFStringRef inKey, CPLAT::CP_ResourcePtr<T>& inOutValue )
{
	bool				result = false;
	CFTypeRef			returnValue;
	
	if( inDictionary != NULL && inKey != NULL )
	{
		inOutValue = ::CFDictionaryGetValue( inDictionary, inKey );
		
		if( inOutValue != NULL )
		{
				// we retrieved the value, now verify the type is correct
			if( ( ::CFGetTypeID( inOutValue ) == CP_CFType_GetTypeID<T>() ) )
			{
					// correct type, so assign to our out value, and set
					// function result to true.
				inOutValue.reset(reinterpret_cast<T>(inOutValue), false);
				
				result = true;
			}
		}
	}
	
	return( result );
}

/*!
*	@function	CP_CFType_Get
*	@brief		Returns in inOutValue a CP_ResourcePtr for the value with the passed in key, and stored
*				in the passed in array.
*/
template <typename T> bool CP_CFType_Get( CFArrayRef inArray, CFIndex inIndex, CPLAT::CP_ResourcePtr<T>& inOutValue )
{
	bool				result = false;
	CFTypeRef			returnValue;
	
	if( inArray != NULL )
	{
		inOutValue = ::CFArrayGetValueAtIndex( inArray, inIndex );
		
		if( inOutValue != NULL )
		{
				// we retrieved the value, now verify the type is correct
			if( ( ::CFGetTypeID( inOutValue ) == CP_CFType_GetTypeID<T>() ) )
			{
					// correct type, so assign to our out value, and set
					// function result to true.
				inOutValue.reset(reinterpret_cast<T>(inOutValue), false);
				
				result = true;
			}
		}
	}
	
	return( result );
}

template <typename T> CPLAT::CP_ResourcePtr<T>
CP_CFType_TypedValue( CFTypeRef inValue )
{
	if( inValue != NULL )
		ASSERT( ::CFGetTypeID(inValue) == CP_CFType_Get<T>() );
	
	return( CPLAT::CP_ResourcePtr<T>(reinterpret_cast<T>( inValue ), false) );
}

CPLAT_End_Namespace_CPLAT

#endif	// _H_CLASS NAME HERE

