//=========================================================================
//
//	FILE:				CP_CarbonEvents.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Definitions for various Carbon Event handling CP_Event
//						classes to handle a specific Carbon Event.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 256 $
//						Latest Checkout:  $Date: 2005-08-26 19:35:37 -0700 (Fri, 26 Aug 2005) $
//
//=========================================================================
#ifndef _H_CP_CarbonEvents
#define _H_CP_CarbonEvents
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC && TARGET_API_MAC_CARBON

	// CPLAT_Framework
#include "CP_Event.h"
#include "CP_Events_Defines.h"
#include "CP_Point.h"
#include "CP_Rect.h"
#include "CP_Region.h"
#include "CP_Size.h"
#include "CP_TypeTraits.h"

CPLAT_Begin_Namespace_CPLAT

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassCommand Events ==
#pragma mark -
#endif

template <> class CP_Event<kEventClassCommand, kEventCommandUpdateStatus> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	HICommand					fHICommand;
};

template <> class CP_Event<kEventClassCommand, kEventCommandProcess> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event();
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	HICommand					fHICommand;
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassControl Events ==
#pragma mark -
#endif

template <> class CP_Event<kEventClassControl, kEventControlInitialize> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}
	
	// class data
	UInt32						fFeatures;	/*! Features used in the initialization. */
};

template <> class CP_Event<kEventClassControl, kEventControlValueFieldChanged> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
};

template <> class CP_Event<kEventClassControl, kCP_Event_Draw> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	CGContextRef				fCGContext;			/*! CoreGraphics port to draw into. */
	CGrafPtr					fGrafPtr;			/*! QuickDraw port to draw into. */
	RgnHandle					fUpdateRegion;		/*! Update region for the event. */

	#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
		HIShapeRef				fHIShape;			/*! HIShapeRef under 10.4 and above under certain circumstances. */
	#endif
};

template <> class CP_Event<kEventClassControl, kEventControlHitTest> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	CP_Point				fMouseLocation;		/*! Mouse location in location coordinates. */
	SInt16					fPartCode;			/*! Part code mouse is over. */
};

template <> class CP_Event<kEventClassControl, kEventControlTrack> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	CP_Point				fMouseLocation;		/*! Mouse location in location coordinates. */
	SInt16					fPartCode;			/*! Part code mouse is over. */
};

template <> class CP_Event<kEventClassControl, kEventControlGetPartRegion> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	RgnHandle				fPartRegion;		/*! Region of the part to return. */
	SInt16					fPartCode;			/*! Part code to return region for. */
};

template <> class CP_Event<kEventClassControl, kEventControlClick> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	// class data	
	CP_Point				fMouseLocation;		/*! Mouse location in global coordinates. */
	UInt32					fClickCount;		/*! Number of time mouse clicked. */

	#if TARGET_RT_MAC_MACHO
		CP_Point			fPortMouseLocation;	/*! Mouse location in port (window relative) coordinates. */
	#endif // TARGET_RT_MAC_MACHO
};

template <> class CP_Event<kEventClassControl, kEventControlGetFocusPart> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	ControlPartCode				fFocusPart;		/*! Part to be focused. */
};

template <> class CP_Event<kEventClassControl, kEventControlSetFocusPart> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	ControlPartCode				fFocusPart;		/*! Part to be focused. */
};

template <> class CP_Event<kEventClassControl, kEventControlContextualMenuClick> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	OSControl					fOSControl;
	CP_Point					fMouseLocation;
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassKeyboard Events ==
#pragma mark -
#endif

template <> class CP_Event<kEventClassKeyboard, kEventRawKeyDown> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	// class data
	char						fCharCode;
	UInt32						fModifiers;
	UInt32						fKeyCode;
	UInt32						fKeyboardType;
};

template <> class CP_Event<kEventClassKeyboard, kEventRawKeyRepeat> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	// class data
	char						fCharCode;
	UInt32						fModifiers;
	UInt32						fKeyCode;
	UInt32						fKeyboardType;
};

template <> class CP_Event<kEventClassKeyboard, kEventRawKeyUp> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	// class data
	char						fCharCode;
	UInt32						fModifiers;
	UInt32						fKeyCode;
	UInt32						fKeyboardType;
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassMouse Events ==
#pragma mark -
#endif

template <> class CP_Event<kEventClassMouse, kEventMouseDown> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	CP_Point				fMouseLocation;
};

template <> class CP_Event<kEventClassMouse, kEventMouseUp> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
};

template <> class CP_Event<kEventClassMouse, kEventMouseMoved> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	CP_Point				fMouseLocation;
	OSWindow				fWindow;
	UInt32					fModifiers;
};

#if TARGET_RT_MAC_MACHO
template <> class CP_Event<kEventClassMouse, kEventMouseEntered> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	CP_Point				fGlobalMouseLocation;
	OSWindow				fWindow;
	UInt32					fModifiers;
	CP_Point				fWindowMouseLocation;
	MouseTrackingRef		fMouseTrackingRef;
};

template <> class CP_Event<kEventClassMouse, kEventMouseExited> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	CP_Point				fGlobalMouseLocation;
	OSWindow				fWindow;
	UInt32					fModifiers;
	CP_Point				fWindowMouseLocation;
	MouseTrackingRef		fMouseTrackingRef;
};
#endif	// TARGET_RT_MAC_MACHO

#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)
template <> class CP_Event<kEventClassControl, kEventControlTrackingAreaEntered> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	CP_Point				fMouseLocation;
};

template <> class CP_Event<kEventClassControl, kEventControlTrackingAreaExited> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	CP_Point				fMouseLocation;
};
#endif	// TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_4) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4)

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassWindow Events ==
#pragma mark -
#endif

template <> class CP_Event<kEventClassWindow, kEventWindowActivated> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	OSWindow					fOSWindow;
};

template <> class CP_Event<kEventClassWindow, kEventWindowDeactivated> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	OSWindow					fOSWindow;
};

template <> class CP_Event<kEventClassWindow, kEventWindowUpdate> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
};

template <> class CP_Event<kEventClassWindow, kEventWindowDrawContent> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
};

template <> class CP_Event<kEventClassWindow, kEventWindowBoundsChanged> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	UInt32						fAttributes;			/*! Attributes of the bounds changed event. */
	CP_Rect						fOriginalBounds;		/*! Original bounds of the event. */
	CP_Rect						fPreviousBounds;		/*! Previous bounds of the event. */
	CP_Rect						fCurrentBounds;			/*! Current bounds of the event. */
};

template <> class CP_Event<kEventClassWindow, kEventWindowClose> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	OSWindow					fOSWindow;
};

template <> class CP_Event<kEventClassWindow, kEventWindowClosed> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	OSWindow					fOSWindow;
};

template <> class CP_Event<kEventClassWindow, kEventWindowGetMinimumSize> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	OSWindow					fOSWindow;
	CP_Size						fMinSize;
};

template <> class CP_Event<kEventClassWindow, kEventWindowGetMaximumSize> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	OSWindow					fOSWindow;
	CP_Size						fMaxSize;
};

template <> class CP_Event<kEventClassWindow, kEventWindowDragCompleted> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	OSWindow					fOSWindow;
};

template <> class CP_Event<kEventClassWindow, kEventWindowContextualMenuSelect> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
	OSWindow					fOSWindow;
	CP_Point					fMouseLocation;
};

#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_2) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2)

template <> class CP_Event<kEventClassWindow, kEventWindowDrawerOpening> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	OSWindow					fOSWindow;
	OSStatus					fReturnStatus;
};

template <> class CP_Event<kEventClassWindow, kEventWindowDrawerOpened> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	OSWindow					fOSWindow;
};

template <> class CP_Event<kEventClassWindow, kEventWindowDrawerClosing> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	OSWindow					fOSWindow;
};

template <> class CP_Event<kEventClassWindow, kEventWindowDrawerClosed> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	OSWindow					fOSWindow;
};

#endif	// TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1020

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassTextInput Events ==
#pragma mark -
#endif

template <> class CP_Event<kEventClassTextInput, kEventTextInputUnicodeForKeyEvent> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//! Destructor.
	virtual						~CP_Event() throw();
	//@}

	// class data
	const CPUniChar				*fUniText;
	UInt32						fTextLength;

private:
	CPUniChar					fFixedSizeBuffer[ 256 ];
	CPUniChar					*fDynamicBuffer;
};

#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_2) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_2)

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassControl 10.2 and above Events ==
#pragma mark -
#endif

#pragma mark CP_Event<kEventClassControl, kEventControlDragEnter>

template <> class CP_Event<kEventClassControl, kEventControlDragEnter> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	bool					fAcceptDrag;
	OSControl				fOSControl;
	DragRef					fDragRef;
};

#pragma mark CP_Event<kEventClassControl, kEventControlDragLeave>

template <> class CP_Event<kEventClassControl, kEventControlDragLeave> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	// class data
	OSControl				fOSControl;
	DragRef					fDragRef;
};

#pragma mark CP_Event<kEventClassControl, kEventControlDragReceive>

template <> class CP_Event<kEventClassControl, kEventControlDragReceive> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	// class data
	OSControl				fOSControl;
	DragRef					fDragRef;
};

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassScrollable Events ==
#pragma mark -
#endif

template <> class CP_Event<kEventClassScrollable, kEventScrollableGetInfo> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}

	//! @name Event Info
	//@{
				void			GetInfo();
				void			ReturnInfo();
	//@}
				
	// class data
	CP_Size						fImageSize;				/*! Size of image. */
	CP_Size						fViewSize;				/*! Size of view displaying image. */
	CP_Point					fImageOrigin;			/*! Origin of the image. */
	CP_Size						fLineSize;				/*! Distance to scroll by line. */
};

template <> class CP_Event<kEventClassScrollable, kEventScrollableScrollTo> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event( CP_Event_Base& inEvent );
	//@}
				
	//! @name Event Info
	//@{
				void			ReturnInfo();
	//@}

	// class data
	CP_Point					fImageLocation;			/*! Top left corner of the image. */
};

#endif	// TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1020

#if TARGET_RT_MAC_MACHO && defined(MAC_OS_X_VERSION_10_3) && (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3)

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === kEventClassTextInput Events ==
#pragma mark -
#endif

template <> class CP_Event<kEventClassTextField, kEventTextAccepted> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_Event( CP_Event_Base& inEvent );
	//@}

	// class data
	OSControl							fEditField;
};

#endif	// TARGET_RT_MAC_MACHO && MAC_OS_X_VERSION_MAX_ALLOWED >= 1030

#if TARGET_CC_METROWERKS
#pragma mark -
#pragma mark === CPLAT Custom Events ==
#pragma mark -
#endif

template <> class CP_Event<CP_E_CP_Event, CP_E_CP_Event_SaveDocument> : public CP_Event_Base
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
								CP_Event();
								CP_Event( CP_Event_Base& inEvent );
	//@}
	
	// class data
};


CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC && TARGET_API_MAC_CARBON

#endif	// _H_CP_CarbonEvents

