//=========================================================================
//
//	FILE:				CP_File_Imp_Mac.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			MacOS implementation of CP_File.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_File_Imp_Mac
#define _H_CP_File_Imp_Mac
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

#if TARGET_OS_MAC

	// CPLAT_Framework
#include "CP_Data_ResourceTypes.h"
#include "CP_Enums.h"
#include "CP_FileSystemSpec.h"
#include "CP_NoCopy.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class	CP_File_Mac_Imp
*	@brief	MacOS implementation of CP_File.
*/
class CP_EXPORT CP_File_Imp : public CP_NoCopy
{
public:
									// Construction / destruction
										CP_File_Imp();
										CP_File_Imp( const CP_FileSystemSpec& inFileSpec );
	virtual								~CP_File_Imp();
			
			void						Create(
												FourCharCode inCreator,
												FourCharCode inType = kCP_Text_FileType );

			void						Open(
												SInt32 inPermissions = CP_E_FilePerm_ReadWrite,
												bool inOpenResourceFork = true );
			SInt16						OpenFork(
											SInt32 inPermissions,
											const HFSUniStr255& inForkName );
			void 						Close();
						
			CP_FileSystemSpec			GetFileSpec()								{ return( fFileSpec ); }
			OSFileRef					GetRefNumber()								{ return( fDataForkRefNum ); }

									// Length
			SInt64						Length();

									// Position
			SInt64						GetPosition() const;
			SInt64						SetPosition(
												UInt16 inMode,
												SInt64 inOffset );
			void 						SetEOF( SInt32 inEOF );

									// Read / Write
			UInt32						ReadData(
												void *inDataBuffer,
												UInt32 inDataSize,
												UInt16 inPositionMode = CP_E_FileSeek_FromCurrent,
												SInt64 inOffset = 0 );
			UInt32						WriteData(
												const void *inDataBuffer,
												UInt32 inDataSize,
												UInt16 inPositionMode = CP_E_FileSeek_FromCurrent,
												SInt64 inOffset = 0 );
		
											// Resource fork
			void						UseResFile();

	//! @name Setters
	//@{
			void						SetFileSpec( const CP_FileSystemSpec& inFileSpec )	{ fFileSpec = inFileSpec; }
	//@}

									// Testers
			bool						IsOpen()											{ return( fDataForkRefNum != kCP_BadFileRefNum ); }							

			void						FlushFork( SInt16 inRefNum );

protected:
			const HFSUniStr255&			GetDataForkName();
			const HFSUniStr255&			GetResourceForkName();

	// class data
protected:
	CP_FileSystemSpec					fFileSpec;				/*! CP_FileSystemSpec containing the name and location of this file. */
	OSFileRef							fDataForkRefNum;		/*! MacOS file reference number for file when open, Data fork. */
	OSFileRef							fResourceForkRefNum;	/*! MacOS file reference number for file when open, Resource fork. */
};

CPLAT_End_Namespace_CPLAT

#endif	// TARGET_OS_MAC

#endif	// _H_CP_File_Imp_Mac

