//=========================================================================
//
//	FILE:				CP_EditField.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms edit field control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 43 $
//						Latest Checkout:  $Date: 2005-01-03 14:42:17 -0800 (Mon, 03 Jan 2005) $
//
//=========================================================================
#ifndef _H_CP_EditField
#define _H_CP_EditField
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_DataValueHolder.h"
#include "CP_FontSpec.h"
#include "CP_KeyInputFilters.h"
#include "CP_PlatformControl.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_EditField
*	@brief		Implementation of a platforms edit field control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_EditField : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_EditField();
										CP_EditField(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inTitle,
												bool inSingleLine,
												bool inIsPassword = false );
	//@}
	//! Destructor.
	virtual								~CP_EditField() throw();
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

	//! @name Getters
	//@{
			CP_String					GetText() const;
	//@}
						
	//! @name Key Filter
	//@{
			void						AttachFilter( CP_KeyInputFilter *inFilter );
			CP_KeyInputFilter			*GetFilter();
	//@}

	//! @name Setters
	//@{
			void						SetText( const CP_String& inText );
			void						SetDisableEditing();
	//@}

	//! @name Slots
	//@{
			void						SetValueSlot(
												CP_DataValueHolder& inValue,
												OSStatus& inOutStatus );
	//@}

	//! @name Value
	//@{
	virtual	void						SetValue( SInt32 inValue );
	virtual	SInt32						GetValue();
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );

	virtual	bool						HandleFieldExitChar(
												char inChar,
												UInt32 inModifiers = 0 );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

private:
void	HandleControlHitEvent( CPLAT::CP_Event_Base& inEvent );

	// class data
protected:
	CP_KeyInputFilter					*fKeyFilter;			// object used to filter our keystrokes
	bool 								fSingleLine;
	
		// signals
	CP_DataValueSignal					fFieldExitSignal;		/*! Signal sent when the user enters a field exit key. */
	
	#if TARGET_OS_MAC
		static ControlEditTextValidationUPP	fsEditFieldValidationProc;
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Events
	//@{
			void						HandleCommandProcess( CP_Event_Base& inEvent );

			void 						HandleRawKeyDown( CP_Event_Base& inEvent );
			void						HandleKeyboardInput_OS9( CP_Event_Base& inEvent );

			void						HandleTextInputEvents( CP_Event_Base& inEvent );

			void						HandleFocusEvents( CP_Event_Base& inEvent );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												bool inSingleLine,
												bool inIsPassword = false );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Events
	//@{
			void 						HandleKeyboardInput_Win( CP_Event_Base& inEvent );
			void 						HandleControlColor( CP_Event_Base& inEvent );

			void						HandleDisabledKeyboardInput_Win( CP_Event_Base& inEvent );
	//@}

	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												bool inSingleLine,
												bool inIsPassword = false );
	//@}
#endif

#if TARGET_OS_LINUX
public:
			void						HandleChangedText(
												gchar *new_text,
												gint new_text_length,
												gint *position );

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												const CP_String& inText,
												bool inSingleLine,
												bool inIsPassword = false );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_EditField

