//=========================================================================
//
//	FILE:				CP_FrameAdapter.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			This class provides support for views that need to be
//						able to reposition and resize themselves when the superview
//						they are containing in resize.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_FrameAdapter
#define _H_CP_FrameAdapter
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_Rect.h"

CPLAT_Begin_Namespace_CPLAT

class CP_EXPORT CP_FrameAdapter
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Destructor.
	virtual								~CP_FrameAdapter() throw();
	//@}

	//! @name Frame management
	//@{
			void						AdjustFrame(
												const CP_Rect& inOriginalFrame,
												const CP_Rect& inNewFrame,
	//@}
										CP_Rect& inAdjustedFrame );		
protected:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FrameAdapter();
	//@}

	//! @name Frame management
	//@{
	virtual	void						HandleAdjustFrame(
												const CP_Rect& inOriginalFrame,
												const CP_Rect& inNewFrame,
												CP_Rect& inAdjustedFrame ) const = 0;
	//@}

	// class data
protected:
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

class CP_EXPORT CP_FrameAdapter_Sticky : public CP_FrameAdapter
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_FrameAdapter_Sticky(
												bool inStickyLeft = true,
												bool inStickyTop = true,
												bool inStickyRight = true,
												bool inStickyBottom = true );
	//! Destructor.
	virtual									~CP_FrameAdapter_Sticky() throw();
	//@}

protected:
	//! @name Frame management
	//@{
	virtual	void						HandleAdjustFrame(
												const CP_Rect& inOriginalFrame,
												const CP_Rect& inNewFrame,
										CP_Rect& inAdjustedFrame ) const;
	//@}

	// class data
protected:
	bool								fStickyLeft;		/*! Flag to indicate if the view is bound to its superviews left side. */
	bool								fStickyTop;			/*! Flag to indicate if the view is bound to its superviews top side. */
	bool								fStickyRight;		/*! Flag to indicate if the view is bound to its superviews right side. */
	bool								fStickyBottom;		/*! Flag to indicate if the view is bound to its superviews bottom side. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
#endif

#if TARGET_OS_WIN32
#endif

#if TARGET_OS_LINUX
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_FrameAdapter

