//=========================================================================
//
//	FILE:				CP_LittleArrowsEditBuddy.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms little arrows control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 2 $
//						Latest Checkout:  $Date: 2004-12-21 19:40:38 -0800 (Tue, 21 Dec 2004) $
//
//=========================================================================
#ifndef _H_CP_LittleArrowsEditBuddy
#define _H_CP_LittleArrowsEditBuddy
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_EditField.h"
#include "CP_LittleArrows.h"
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

/*!
*	@class		CP_LittleArrowsEditBuddy
*	@brief		Implementation of a platforms little arrows control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_LittleArrowsEditBuddy : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_LittleArrowsEditBuddy();
										CP_LittleArrowsEditBuddy(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												UInt32 inCommand,
												SInt32 inIncrement = 1,
												bool inAlignRight = true );
	//! Destructor.
	virtual								~CP_LittleArrowsEditBuddy() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
	//! @name Value
	//@{
	virtual	SInt32						GetValue();
	virtual	void						SetValue( SInt32 inValue );
			
	virtual	SInt32						GetMin();
	virtual	void						SetMin( SInt32 inValue );

	virtual	SInt32						GetMax();
	virtual	void						SetMax( SInt32 inValue );
	//@}

protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

	// class data
protected:
	
	#if TARGET_OS_MAC
		CP_LittleArrows					*fArrows;
		CP_EditField					*fBuddyEdit;
	#endif
	
	#if TARGET_OS_WIN32
		OSControl						fEditFieldControl;
	#endif
	
	SInt32								fValue;
	SInt32								fMaximum;
	SInt32								fMinimum;

		// Signals
	CP_DataValueSignal					fValueChangedSignal;	/*! Signal sent when the value changes. */
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												UInt32 inCommand,
												SInt32 inIncrement = 1,
												bool inAlignRight = true );
	//@}

			void						HandleReturn(
												CP_DataValueHolder& inDataValue,
												OSStatus& inOutOSStatus );
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												UInt32 inCommand,
												SInt32 inIncrement = 1,
												bool inAlignRight = true );
	//@}

	//! @name Initialization
	//@{
			void						HandleValueChangedEvent( CP_Event_Base& inEvent );
	//@}

			void 						SetBuddyFieldValue(
												CP_DataValueHolder& inDataValue,
												OSStatus& inOutOSStatus );

#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize,
												SInt32 inValue,
												SInt32 inMinimum,
												SInt32 inMaximum,
												UInt32 inCommand,
												SInt32 inIncrement = 1,
												bool inAlignRight = true );
	//@}
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_LittleArrowsEditBuddy
