//=========================================================================
//
//	FILE:				CP_PrintView.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			CP_View class that holds other pane and views when
//						printing. You use this class to manage the panes and
//						views you want to use when printing out a page. The
//						different panes and views can be different size and
//						don't have to be displayed in a CP_Window object to
//						use in this view.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 80 $
//						Latest Checkout:  $Date: 2005-02-08 15:25:15 -0800 (Tue, 08 Feb 2005) $
//
//=========================================================================
#ifndef _H_CP_PrintView
#define _H_CP_PrintView
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_View.h"

CPLAT_Begin_Namespace_CPLAT

//!  CP_PrintView
/*!
  CP_View class that holds other pane and views when printing. You use this
  class to manage the panes and views you want to use when printing out a
  page. The different panes and views can be different size and don't have
  to be displayed in a CP_Window object to use in this view.
*/
class CP_EXPORT CP_PrintView : public CP_View
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_PrintView();
										CP_PrintView(
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_PrintView() throw();
	//@}
			
	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}

											// Printing
	virtual	void						DoPrint(
												CP_Printer *inPrinter,
												CP_Window *inParentWindow = NULL );

											// Subview handling
			void						AddPrintingView(
												CP_View *inSubview,
												CP_Point& inSizeLocation,
												CP_Size& inPrintSize );
protected:
	virtual	void						GetMaximumTilesNeeded(
												CP_Printer *inPrinter,
												CP_Printer_PageFormat& inPageFormat,
												UInt16 *inOutHorTiles,
												UInt16 *inOutVerTiles );

	// class data
protected:

//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
protected:
	virtual	void						DoOSPrintLoop(
												CP_Printer *inPrinter,
												CP_Window *inParentWindow );
#endif

#if TARGET_OS_WIN32
protected:
	virtual	void						DoOSPrintLoop(
												CP_Printer *inPrinter,
												CP_Window *inParentWindow );

private:
			void						RegisterWindowClass();
#endif

#if TARGET_OS_LINUX
protected:
	virtual	void						DoOSPrintLoop(
												CP_Printer *inPrinter,
												CP_Window *inParentWindow );
#endif
};

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_PrintView
