//=========================================================================
//
//	FILE:				CP_TableHeaders.h
//	
//						Copyright  2004. Ken Stahlman. All Rights Reserved.
//	
//	PURPOSE:			Implementation of a platforms window list header control.
//	
//	HISTORY:			1st Version - August 2004.
//	
//	NOTES:				- none -
//
//	VERSION:			Current Revision: $Rev: 127 $
//						Latest Checkout:  $Date: 2005-03-26 13:06:44 -0800 (Sat, 26 Mar 2005) $
//
//=========================================================================
#ifndef _H_CP_TableHeaders
#define _H_CP_TableHeaders
#ifndef TARGET_OS_LINUX
	#pragma once
#endif

	// CPLAT_Framework
#include "CP_PlatformControl.h"

CPLAT_Begin_Namespace_CPLAT

// ===================================================================================
//	CP_S_TableHeaderItemInfo - structure passed into our window list header routines with
//		information about a column in the list.
// ===================================================================================
struct CP_S_TableHeaderItemInfo
{
	SInt16					fImageResID;
	
	SInt16					fColumnWidth;
    SInt16					fColumnMinSize;			// minimum size column can go
	
	UInt32					fCommandID;				// command sent when clicked on
	
	SInt32					fUserData;
	SInt32					fTextAlign;
	CP_String				fText;
	
							CP_S_TableHeaderItemInfo();
							CP_S_TableHeaderItemInfo(
									const CP_String& inText,
									UInt32 inCommandID,
									SInt32 inSInt32 = CP_E_TextAlign_HorLeft,
									const SInt16 inImageResID = kCP_NoImageIndex,
									const SInt16 inColumnWidth = 10,
									SInt32 inUserData = 0,
									SInt16 inColMinSize = 1 );
							CP_S_TableHeaderItemInfo( const CP_S_TableHeaderItemInfo& inTabProperty );

	CP_S_TableHeaderItemInfo&	operator =( const CP_S_TableHeaderItemInfo& inTabProperty );
};

#define kCP_TableHeadersEdgeOffset			6

	// typedefs
#if TARGET_OS_LINUX
	typedef std::vector<CP_S_TableHeaderItemInfo, std::allocator<CP_S_TableHeaderItemInfo> >			CP_TableHeaderItem_Array;
	typedef std::vector<CP_S_TableHeaderItemInfo, std::allocator<CP_S_TableHeaderItemInfo> >::iterator	CP_TableHeaderItem_Iterator;
#else
	typedef std::vector<CP_S_TableHeaderItemInfo >														CP_TableHeaderItem_Array;
	typedef std::vector<CP_S_TableHeaderItemInfo >::iterator											CP_TableHeaderItem_Iterator;
#endif

/*!
*	@class		CP_TableHeaders
*	@brief		Implementation of a platforms window list header control.
*
*	@ingroup	UI
*/
class CP_EXPORT CP_TableHeaders : public CP_PlatformControl
{
public:
	//! @name Constructor / Destructor
	//@{
	//! Constructor.
										CP_TableHeaders();
										CP_TableHeaders(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//! Destructor.
	virtual								~CP_TableHeaders() throw();
	//@}
			
	//! @name Items
	//@{
			void						AddItem( CP_S_TableHeaderItemInfo& inItemInfo );
			void						InsertItem(
												CP_S_TableHeaderItemInfo& inItemInfo,
												UInt32 inIndex );
			void						DeleteItem( UInt32 inIndex );

			void						SetItem(
												const CP_S_TableHeaderItemInfo& inItemInfo,
												UInt32 inIndex );
			CP_S_TableHeaderItemInfo	GetItem( const UInt32 inColumn );

			UInt32						GetItemCount();
	//@}

	//! @name Persistance
	//@{
	static CP_M_Persist					*PersistCreator();
	//@}
			
protected:
	//! @name Events
	//@{
	virtual	void						InitEventHandler( CP_EventHandler& inEventHandler );
	//@}

	//! @name Persistance
	//@{
	virtual	void						OnInitPersist(
												CP_View *inSuperview,
												CP_DataValueList& inDataValueList );
	//@}

private:
	//! @name Slots
	//@{
			void						HandleControlHitEvent( CP_Event_Base& inEvent );
			void						HandleMouseUpEvent( CP_Event_Base& inEvent );
	//@}

	//! @name Events
	//@{
#if ! TARGET_OS_WIN32
			void						HandleClickInColumn( const CP_Point& inLocalMouseClick );
#endif
	//@}

	//! @name Misc
	//@{
			SInt16						FindColumnFromPoint( const CP_Point& inLocalMouseClick );
	//@}

	// class data
protected:
	CP_TableHeaderItem_Array			*fItems;
	
	CP_DataValueSignal					fColumnResizingSignal;		/*! Signal sent when a column is resizing. */
	CP_DataValueSignal					fColumnSortSignal;			/*! Signal sent when a column is clicked on to optionally sort the tables data. */

	bool								fSortDirectionUp;
	
	#if TARGET_OS_MAC
		SInt32							fInitialColResizing;		/*! Used when a click occurs in the column
																		headers in report view for column resizing. */
	#endif
	
//========================================================================
// Platform specific routines
//========================================================================
#if TARGET_OS_MAC
public:
	virtual	void						HandleColumnResizing(
												const CP_Point& inMouseLocation,
												void *inData );

protected:
	//! @name Drawing
	//@{
	virtual	void						OnDraw(
												CP_Canvas& inCanvas,
												CP_Region& inUpdateArea );

			void						DrawOneItem(
												CP_Canvas& inCanvas,
												CP_Rect& inBounds,
												const CP_S_TableHeaderItemInfo& inColumnInfo,
												bool inIsSelected,
												bool inIsPressed );
			void						CalcRects(
												CP_Canvas& inCanvas,
												CP_Rect& inBounds,
												const CP_S_TableHeaderItemInfo& inColumnInfo,
												CP_Rect& inOutTextBounds,
												CP_Rect& inOutImageBounds );
			void						DrawColumnDivider(
												CP_Canvas& inCanvas,
												const CP_Rect& inBounds );
	//@}

private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Column Header Clicks
	//@{
			bool						IsPointInColHeaderBorder( const CP_Point& inLocation );
			void						HandleClickInColHeaders( const CP_Point& inLocalPoint );
	//@}
#endif

#if TARGET_OS_WIN32
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}

	//! @name Initialization
	//@{
			void						HandleValueChangedEvent( CP_Event_Base& inEvent );
	//@}
#endif

#if TARGET_OS_LINUX
private:
	//! @name Initialization
	//@{
			void						CreatePlatformImp(
												CP_View *inSuperview,
												const CP_Point& inUpperLeft,
												const CP_Size& inContentSize );
	//@}
#endif
};

#if TARGET_OS_MAC || TARGET_OS_LINUX
		// mouse tracking callbacks
	void CP_WindowListHeader_ColumnResize_TrackMouse( CP_View *inDestView, CP_Point& inMouseLocation, void *inData );

	typedef struct CP_S_HeaderColumnTracking
	{
		CP_TableHeaders			*fTableHeadersPtr;		// pointer to the CP_TableHeaders
		SInt16					fColToResize;			// column mouse is to resize
		SInt16					fColStartHorPosn;		// the starting position of the column header
	} CP_S_HeaderColumnTracking;
#endif

CPLAT_End_Namespace_CPLAT

#endif	// _H_CP_TableHeaders
